@@ -80,6 +80,15 @@ class HANLINV3(USBMS):
             drives['carda'] = main
         return drives
 
+class SPECTRA(HANLINV3):
+
+    name = 'Spectra'
+    gui_name = 'Spectra'
+    PRODUCT_ID  = [0xa4a5]
+
+    FORMATS = ['epub', 'mobi', 'fb2', 'lit', 'prc', 'chm', 'djvu', 'pdf', 'rtf', 'txt']
+
+    SUPPORTS_SUB_DIRS = True
 
 class HANLINV5(HANLINV3):
     name           = 'Hanlin V5 driver'
@@ -106,9 +115,12 @@ class BOOX(HANLINV3):
     description    = _('Communicate with the BOOX eBook reader.')
     author         = 'Jesus Manuel Marinho Valcarce'
     supported_platforms = ['windows', 'osx', 'linux']
+    METADATA_CACHE = '.metadata.calibre'
+    DRIVEINFO = '.driveinfo.calibre'
 
     # Ordered list of supported formats
-    FORMATS     = ['epub', 'fb2', 'djvu', 'pdf', 'html', 'txt', 'rtf', 'mobi', 'prc', 'chm']
+    FORMATS     = ['epub', 'fb2', 'djvu', 'pdf', 'html', 'txt', 'rtf', 'mobi',
+                   'prc', 'chm', 'doc']
 
     VENDOR_ID   = [0x0525]
     PRODUCT_ID  = [0xa4a5]
