@@ -35,6 +35,16 @@ class DevicePlugin(Plugin):
 
     #: Height for thumbnails on the device
     THUMBNAIL_HEIGHT = 68
+    #: Width for thumbnails on the device. Setting this will force thumbnails
+    #: to this size, not preserving aspect ratio. If it is not set, then
+    #: the aspect ratio will be preserved and the thumbnail will be no higher
+    #: than THUMBNAIL_HEIGHT
+    # THUMBNAIL_WIDTH = 68
+
+    #: Set this to True if the device supports updating cover thumbnails during
+    #: sync_booklists. Setting it to true will ask device.py to refresh the
+    #: cover thumbnails during book matching
+    WANTS_UPDATED_THUMBNAILS = False
 
     #: Whether the metadata on books can be set via the GUI.
     CAN_SET_METADATA = ['title', 'authors', 'collections']
@@ -205,7 +215,7 @@ class DevicePlugin(Plugin):
 
         return True
 
-    def open(self):
+    def open(self, library_uuid):
         '''
         Perform any device specific initialization. Called after the device is
         detected but before any other functions that communicate with the device.
@@ -250,6 +260,8 @@ class DevicePlugin(Plugin):
         Ask device for device information. See L{DeviceInfoQuery}.
 
         :return: (device name, device version, software version on device, mime type)
+                 The tuple can optionally have a fifth element, which is a
+                 drive information diction. See usbms.driver for an example.
 
         """
         raise NotImplementedError()
@@ -437,6 +449,15 @@ class DevicePlugin(Plugin):
         '''
         pass
 
+    def set_driveinfo_name(self, location_code, name):
+        '''
+        Set the device name in the driveinfo file to 'name'. This setting will
+        persist until the file is re-created or the name is changed again.
+
+        Non-disk devices will ignore this request.
+        '''
+        pass
+
 class BookList(list):
     '''
     A list of books. Each Book object must have the fields
