@@ -22,7 +22,7 @@ class KOBO(USBMS):
     gui_name = 'Kobo Reader'
     description = _('Communicate with the Kobo Reader')
     author = 'Timothy Legge and Kovid Goyal'
-    version = (1, 0, 7)
+    version = (1, 0, 9)
 
     dbversion = 0
     fwversion = 0
@@ -78,9 +78,13 @@ class KOBO(USBMS):
                  else self._main_prefix
 
         # Determine the firmware version
-        f = open(self.normalize_path(self._main_prefix + '.kobo/version'), 'r')
-        self.fwversion = f.readline().split(',')[2]
-        f.close()
+        try:
+            with open(self.normalize_path(self._main_prefix + '.kobo/version'),
+                    'rb') as f:
+                self.fwversion = f.readline().split(',')[2]
+        except:
+            self.fwversion = 'unknown'
+
         if self.fwversion != '1.0' and self.fwversion != '1.4':
             self.has_kepubs = True
         debug_print('Version of firmware: ', self.fwversion, 'Has kepubs:', self.has_kepubs)
@@ -98,7 +102,6 @@ class KOBO(USBMS):
 
         def update_booklist(prefix, path, title, authors, mime, date, ContentType, ImageID, readstatus, MimeType):
             changed = False
-            # if path_to_ext(path) in self.FORMATS:
             try:
                 lpath = path.partition(self.normalize_path(prefix))[2]
                 if lpath.startswith(os.sep):
@@ -112,6 +115,8 @@ class KOBO(USBMS):
                     playlist_map[lpath]= "Im_Reading"
                 elif readstatus == 2:
                     playlist_map[lpath]= "Read"
+                elif readstatus == 3:
+                    playlist_map[lpath]= "Closed"
 
                 path = self.normalize_path(path)
                 # print "Normalized FileName: " + path
@@ -125,9 +130,12 @@ class KOBO(USBMS):
                         if imagename is not None:
                             bl[idx].thumbnail = ImageWrapper(imagename)
                     if (ContentType != '6' and MimeType != 'Shortcover'):
-                        if self.update_metadata_item(bl[idx]):
-                            # print 'update_metadata_item returned true'
-                            changed = True
+                        if os.path.exists(self.normalize_path(os.path.join(prefix, lpath))):
+                            if self.update_metadata_item(bl[idx]):
+                                # print 'update_metadata_item returned true'
+                                changed = True
+                        else:
+                             debug_print("    Strange:  The file: ", prefix, lpath, " does mot exist!")
                     if lpath in playlist_map and \
                         playlist_map[lpath] not in bl[idx].device_collections:
                             bl[idx].device_collections.append(playlist_map[lpath])
@@ -136,7 +144,13 @@ class KOBO(USBMS):
                         book =  Book(prefix, lpath, title, authors, mime, date, ContentType, ImageID, size=1048576)
                     else:
                         try:
-                            book = self.book_from_path(prefix, lpath, title, authors, mime, date, ContentType, ImageID)
+                            if os.path.exists(self.normalize_path(os.path.join(prefix, lpath))):
+                                book = self.book_from_path(prefix, lpath, title, authors, mime, date, ContentType, ImageID)
+                            else:
+                                debug_print("    Strange:  The file: ", prefix, lpath, " does mot exist!")
+                                title = "FILE MISSING: " + title
+                                book =  Book(prefix, lpath, title, authors, mime, date, ContentType, ImageID, size=1048576)
+
                         except:
                             debug_print("prefix: ", prefix, "lpath: ", lpath, "title: ", title, "authors: ", authors, \
                                         "mime: ", mime, "date: ", date, "ContentType: ", ContentType, "ImageID: ", ImageID)
@@ -153,6 +167,10 @@ class KOBO(USBMS):
             return changed
 
         connection = sqlite.connect(self.normalize_path(self._main_prefix + '.kobo/KoboReader.sqlite'))
+
+        # return bytestrings if the content cannot the decoded as unicode
+        connection.text_factory = lambda x: unicode(x, "utf-8", "ignore")
+
         cursor = connection.cursor()
 
         #query = 'select count(distinct volumeId) from volume_shortcovers'
@@ -220,8 +238,12 @@ class KOBO(USBMS):
         #    2) volume_shorcover
         #    2) content
 
-        debug_print('delete_via_sql: ContentID: ', ContentID, 'ContentType: ', ContentType) 
+        debug_print('delete_via_sql: ContentID: ', ContentID, 'ContentType: ', ContentType)
         connection = sqlite.connect(self.normalize_path(self._main_prefix + '.kobo/KoboReader.sqlite'))
+
+        # return bytestrings if the content cannot the decoded as unicode
+        connection.text_factory = lambda x: unicode(x, "utf-8", "ignore")
+
         cursor = connection.cursor()
         t = (ContentID,)
         cursor.execute('select ImageID from content where ContentID = ?', t)
@@ -495,6 +517,10 @@ class KOBO(USBMS):
         # the last book from the collection the list of books is empty
         # and the removal of the last book would not occur
         connection = sqlite.connect(self.normalize_path(self._main_prefix + '.kobo/KoboReader.sqlite'))
+
+        # return bytestrings if the content cannot the decoded as unicode
+        connection.text_factory = lambda x: unicode(x, "utf-8", "ignore")
+
         cursor = connection.cursor()
 
 
@@ -532,7 +558,7 @@ class KOBO(USBMS):
                         if result is None:
                             datelastread = '1970-01-01T00:00:00'
                         else:
-                            datelastread = result[0] if result[0] is not None else '1970-01-01T00:00:00' 
+                            datelastread = result[0] if result[0] is not None else '1970-01-01T00:00:00'
 
                         t = (datelastread,ContentID,)
 
@@ -575,11 +601,47 @@ class KOBO(USBMS):
                         try:
                             cursor.execute('update content set ReadStatus=2,FirstTimeReading=\'true\' where BookID is Null and ContentID = ?', t)
                         except:
-                            debug_print('Database Exception:  Unable set book as Rinished')
+                            debug_print('Database Exception:  Unable set book as Finished')
                             raise
                         else:
                             connection.commit()
 #                            debug_print('Database: Commit set ReadStatus as Finished')
+                if category == 'Closed':
+                    # Reset Im_Reading list in the database
+                    if oncard == 'carda':
+                        query= 'update content set ReadStatus=0, FirstTimeReading = \'true\' where BookID is Null and ReadStatus = 3 and ContentID like \'file:///mnt/sd/%\''
+                    elif oncard != 'carda' and oncard != 'cardb':
+                        query= 'update content set ReadStatus=0, FirstTimeReading = \'true\' where BookID is Null and ReadStatus = 3 and ContentID not like \'file:///mnt/sd/%\''
+
+                    try:
+                        cursor.execute (query)
+                    except:
+                        debug_print('Database Exception:  Unable to reset Closed list')
+                        raise
+                    else:
+#                       debug_print('Commit: Reset Closed list')
+                        connection.commit()
+
+                    for book in books:
+#                       debug_print('Title:', book.title, 'lpath:', book.path)
+                        book.device_collections = ['Closed']
+
+                        extension =  os.path.splitext(book.path)[1]
+                        ContentType = self.get_content_type_from_extension(extension) if extension != '' else self.get_content_type_from_path(book.path)
+
+                        ContentID = self.contentid_from_path(book.path, ContentType)
+#                        datelastread = time.strftime("%Y-%m-%dT%H:%M:%S", time.gmtime())
+
+                        t = (ContentID,)
+
+                        try:
+                            cursor.execute('update content set ReadStatus=3,FirstTimeReading=\'true\' where BookID is Null and ContentID = ?', t)
+                        except:
+                            debug_print('Database Exception:  Unable set book as Closed')
+                            raise
+                        else:
+                            connection.commit()
+#                            debug_print('Database: Commit set ReadStatus as Closed')
         else: # No collections
             # Since no collections exist the ReadStatus needs to be reset to 0 (Unread)
             print "Reseting ReadStatus to 0"
