@@ -98,6 +98,9 @@ class Device(DeviceConfig, DevicePlugin):
     # copy these back to the library
     BACKLOADING_ERROR_MESSAGE = None
 
+    #: The maximum length of paths created on the device
+    MAX_PATH_LEN = 250
+
     def reset(self, key='-1', log_packets=False, report_progress=None,
             detected_device=None):
         self._main_prefix = self._card_a_prefix = self._card_b_prefix = None
@@ -229,16 +232,37 @@ class Device(DeviceConfig, DevicePlugin):
 
         time.sleep(5)
         drives = {}
+        seen = set()
+        prod_pat = re.compile(r'PROD_(.+?)&')
+        dup_prod_id = False
+
+        def check_for_dups(pnp_id):
+            try:
+                match = prod_pat.search(pnp_id)
+                if match is not None:
+                    prodid = match.group(1)
+                    if prodid in seen:
+                        return True
+                    else:
+                        seen.add(prodid)
+            except:
+                pass
+            return False
+
+
         for drive, pnp_id in win_pnp_drives().items():
             if self.windows_match_device(pnp_id, 'WINDOWS_CARD_A_MEM') and \
                     not drives.get('carda', False):
                 drives['carda'] = drive
+                dup_prod_id |= check_for_dups(pnp_id)
             elif self.windows_match_device(pnp_id, 'WINDOWS_CARD_B_MEM') and \
                     not drives.get('cardb', False):
                 drives['cardb'] = drive
+                dup_prod_id |= check_for_dups(pnp_id)
             elif self.windows_match_device(pnp_id, 'WINDOWS_MAIN_MEM') and \
                     not drives.get('main', False):
                 drives['main'] = drive
+                dup_prod_id |= check_for_dups(pnp_id)
 
             if 'main' in drives.keys() and 'carda' in drives.keys() and \
                     'cardb' in drives.keys():
@@ -260,7 +284,8 @@ class Device(DeviceConfig, DevicePlugin):
 
         # Sort drives by their PNP drive numbers if the CARD and MAIN
         # MEM strings are identical
-        if self.WINDOWS_MAIN_MEM in (self.WINDOWS_CARD_A_MEM,
+        if dup_prod_id or \
+                self.WINDOWS_MAIN_MEM in (self.WINDOWS_CARD_A_MEM,
                 self.WINDOWS_CARD_B_MEM) or \
                 self.WINDOWS_CARD_A_MEM == self.WINDOWS_CARD_B_MEM:
             letters = sorted(drives.values(), cmp=drivecmp)
@@ -675,7 +700,7 @@ class Device(DeviceConfig, DevicePlugin):
 
 
 
-    def open(self):
+    def open(self, library_uuid):
         time.sleep(5)
         self._main_prefix = self._card_a_prefix = self._card_b_prefix = None
         if islinux:
@@ -697,6 +722,7 @@ class Device(DeviceConfig, DevicePlugin):
                 time.sleep(7)
                 self.open_osx()
 
+        self.current_library_uuid = library_uuid
         self.post_open_callback()
 
     def post_open_callback(self):
@@ -875,7 +901,7 @@ class Device(DeviceConfig, DevicePlugin):
 
     def create_upload_path(self, path, mdata, fname, create_dirs=True):
         path = os.path.abspath(path)
-        extra_components = []
+        maxlen = self.MAX_PATH_LEN
 
         special_tag = None
         if mdata.tags:
@@ -902,7 +928,7 @@ class Device(DeviceConfig, DevicePlugin):
         app_id = str(getattr(mdata, 'application_id', ''))
         # The db id will be in the created filename
         extra_components = get_components(template, mdata, fname,
-                timefmt=opts.send_timefmt, length=250-len(app_id)-1)
+                timefmt=opts.send_timefmt, length=maxlen-len(app_id)-1)
         if not extra_components:
             extra_components.append(sanitize(self.filename_callback(fname,
                 mdata)))
@@ -937,12 +963,11 @@ class Device(DeviceConfig, DevicePlugin):
             return ans
 
         extra_components = list(map(remove_trailing_periods, extra_components))
-        components = shorten_components_to(250 - len(path), extra_components)
+        components = shorten_components_to(maxlen - len(path), extra_components)
         components = self.sanitize_path_components(components)
         filepath = os.path.join(path, *components)
         filedir = os.path.dirname(filepath)
 
-
         if create_dirs and not os.path.exists(filedir):
             os.makedirs(filedir)
 
