@@ -131,9 +131,12 @@ class PageProcessor(list): # {{{
                     newsizey = int(newsizex / aspect)
                     deltax = 0
                     deltay = (SCRHEIGHT - newsizey) / 2
-                wand.size = (newsizex, newsizey)
-                wand.set_border_color(pw)
-                wand.add_border(pw, deltax, deltay)
+                if newsizex < 20000 and newsizey < 20000:
+                    # Too large and resizing fails, so better
+                    # to leave it as original size
+                    wand.size = (newsizex, newsizey)
+                    wand.set_border_color(pw)
+                    wand.add_border(pw, deltax, deltay)
             elif self.opts.wide:
                 # Keep aspect and Use device height as scaled image width so landscape mode is clean
                 aspect = float(sizex) / float(sizey)
@@ -152,11 +155,15 @@ class PageProcessor(list): # {{{
                     newsizey = int(newsizex / aspect)
                     deltax = 0
                     deltay = (wscreeny - newsizey) / 2
-                wand.size = (newsizex, newsizey)
-                wand.set_border_color(pw)
-                wand.add_border(pw, deltax, deltay)
+                if newsizex < 20000 and newsizey < 20000:
+                    # Too large and resizing fails, so better
+                    # to leave it as original size
+                    wand.size = (newsizex, newsizey)
+                    wand.set_border_color(pw)
+                    wand.add_border(pw, deltax, deltay)
             else:
-                wand.size = (SCRWIDTH, SCRHEIGHT)
+                if SCRWIDTH < 20000 and SCRHEIGHT < 20000:
+                    wand.size = (SCRWIDTH, SCRHEIGHT)
 
             if not self.opts.dont_sharpen:
                 wand.sharpen(0.0, 1.0)
@@ -304,6 +311,10 @@ class ComicInput(InputFormatPlugin):
             help=_('Specify the image size as widthxheight pixels. Normally,'
                 ' an image size is automatically calculated from the output '
                 'profile, this option overrides it.')),
+        OptionRecommendation(name='dont_add_comic_pages_to_toc', recommended_value=False,
+            help=_('When converting a CBC do not add links to each page to'
+                ' the TOC. Note this only applies if the TOC has more than one'
+                ' section')),
         ])
 
     recommendations = set([
@@ -449,10 +460,11 @@ class ComicInput(InputFormatPlugin):
                 wrappers = comic[2]
                 stoc = toc.add_item(href(wrappers[0]),
                         None, comic[0], play_order=po)
-                for i, x in enumerate(wrappers):
-                    stoc.add_item(href(x), None,
-                            _('Page')+' %d'%(i+1), play_order=po)
-                    po += 1
+                if not opts.dont_add_comic_pages_to_toc:
+                    for i, x in enumerate(wrappers):
+                        stoc.add_item(href(x), None,
+                                _('Page')+' %d'%(i+1), play_order=po)
+                        po += 1
         opf.set_toc(toc)
         m, n = open('metadata.opf', 'wb'), open('toc.ncx', 'wb')
         opf.render(m, n, 'toc.ncx')
