@@ -8,18 +8,37 @@ __docformat__ = 'restructuredtext en'
 import os
 from functools import partial
 
-from PyQt4.Qt import QPixmap, QMenu
+from PyQt4.Qt import QPixmap, QMenu, QTimer
 
 
 from calibre.gui2 import error_dialog, choose_files, \
     choose_dir, warning_dialog, info_dialog
 from calibre.gui2.dialogs.add_empty_book import AddEmptyBookDialog
+from calibre.gui2.dialogs.progress import ProgressDialog
 from calibre.gui2.widgets import IMAGE_EXTENSIONS
 from calibre.ebooks import BOOK_EXTENSIONS
 from calibre.utils.filenames import ascii_filename
 from calibre.constants import preferred_encoding, filesystem_encoding
 from calibre.gui2.actions import InterfaceAction
-from calibre.gui2 import config
+from calibre.gui2 import config, question_dialog
+from calibre.ebooks.metadata import MetaInformation
+
+def get_filters():
+    return [
+            (_('Books'), BOOK_EXTENSIONS),
+            (_('EPUB Books'), ['epub']),
+            (_('LRF Books'), ['lrf']),
+            (_('HTML Books'), ['htm', 'html', 'xhtm', 'xhtml']),
+            (_('LIT Books'), ['lit']),
+            (_('MOBI Books'), ['mobi', 'prc', 'azw']),
+            (_('Topaz books'), ['tpz','azw1']),
+            (_('Text books'), ['txt', 'rtf']),
+            (_('PDF Books'), ['pdf']),
+            (_('SNB Books'), ['snb']),
+            (_('Comics'), ['cbz', 'cbr', 'cbc']),
+            (_('Archives'), ['zip', 'rar']),
+    ]
+
 
 class AddAction(InterfaceAction):
 
@@ -45,6 +64,10 @@ class AddAction(InterfaceAction):
         self.add_menu.addAction(_('Add Empty book. (Book entry with no '
             'formats)'), self.add_empty, _('Shift+Ctrl+E'))
         self.add_menu.addAction(_('Add from ISBN'), self.add_from_isbn)
+        self.add_menu.addSeparator()
+        self.add_menu.addAction(_('Add files to selected book records'),
+                self.add_formats, _('Shift+A'))
+
         self.qaction.setMenu(self.add_menu)
         self.qaction.triggered.connect(self.add_books)
 
@@ -53,6 +76,39 @@ class AddAction(InterfaceAction):
         for action in list(self.add_menu.actions())[1:]:
             action.setEnabled(enabled)
 
+    def add_formats(self, *args):
+        if self.gui.stack.currentIndex() != 0:
+            return
+        view = self.gui.library_view
+        rows = view.selectionModel().selectedRows()
+        if not rows:
+            return
+        ids = [view.model().id(r) for r in rows]
+
+        if len(ids) > 1 and not question_dialog(self.gui,
+                _('Are you sure'),
+            _('Are you sure you want to add the same'
+                ' files to all %d books? If the format'
+                'already exists for a book, it will be replaced.')%len(ids)):
+                return
+
+        books = choose_files(self.gui, 'add formats dialog dir',
+                _('Select book files'), filters=get_filters())
+        if not books:
+            return
+
+        db = view.model().db
+        for id_ in ids:
+            for fpath in books:
+                fmt = os.path.splitext(fpath)[1][1:].upper()
+                if fmt:
+                    db.add_format_with_hooks(id_, fmt, fpath, index_is_id=True,
+                        notify=True)
+        current_idx = self.gui.library_view.currentIndex()
+        if current_idx.isValid():
+            view.model().current_changed(current_idx, current_idx)
+
+
     def add_recursive(self, single):
         root = choose_dir(self.gui, 'recursive book import root dir dialog',
                           'Select root folder')
@@ -95,7 +151,6 @@ class AddAction(InterfaceAction):
         dlg = AddEmptyBookDialog(self.gui, self.gui.library_view.model().db, author)
         if dlg.exec_() == dlg.Accepted:
             num = dlg.qty_to_add
-            from calibre.ebooks.metadata import MetaInformation
             for x in xrange(num):
                 mi = MetaInformation(_('Unknown'), dlg.selected_authors)
                 self.gui.library_view.model().db.import_book(mi, [])
@@ -105,41 +160,73 @@ class AddAction(InterfaceAction):
             self.gui.tags_view.recount()
 
     def add_isbns(self, books, add_tags=[]):
-        from calibre.ebooks.metadata import MetaInformation
-        ids = set([])
-        db = self.gui.library_view.model().db
+        self.isbn_books = list(books)
+        self.add_by_isbn_ids = set()
+        self.isbn_add_tags = add_tags
+        QTimer.singleShot(10, self.do_one_isbn_add)
+        self.isbn_add_dialog = ProgressDialog(_('Adding'),
+                _('Creating book records from ISBNs'), max=len(books),
+                cancelable=False, parent=self.gui)
+        self.isbn_add_dialog.exec_()
+
+    def do_one_isbn_add(self):
+        try:
+            db = self.gui.library_view.model().db
+
+            try:
+                x = self.isbn_books.pop(0)
+            except IndexError:
+                self.gui.library_view.model().books_added(self.isbn_add_dialog.value)
+                self.isbn_add_dialog.accept()
+                orig = config['overwrite_author_title_metadata']
+                config['overwrite_author_title_metadata'] = True
+                try:
+                    self.gui.iactions['Edit Metadata'].do_download_metadata(
+                            self.add_by_isbn_ids)
+                finally:
+                    config['overwrite_author_title_metadata'] = orig
+                return
+
 
-        for x in books:
             mi = MetaInformation(None)
             mi.isbn = x['isbn']
-            if x['path'] is not None:
-                ids.add(db.import_book(mi, [x['path']]))
-            else:
-                ids.add(db.import_book(mi, []))
-        self.gui.library_view.model().books_added(len(books))
-        orig = config['overwrite_author_title_metadata']
-        config['overwrite_author_title_metadata'] = True
-        try:
-            self.gui.iactions['Edit Metadata'].do_download_metadata(ids)
-        finally:
-            config['overwrite_author_title_metadata'] = orig
-        if add_tags and ids:
-            db.bulk_modify_tags(ids, add=add_tags)
-
+            if self.isbn_add_tags:
+                mi.tags = list(self.isbn_add_tags)
+            fmts = [] if x['path'] is None else [x['path']]
+            self.add_by_isbn_ids.add(db.import_book(mi, fmts))
+            self.isbn_add_dialog.value += 1
+            QTimer.singleShot(10, self.do_one_isbn_add)
+        except:
+            self.isbn_add_dialog.accept()
+            raise
 
     def files_dropped(self, paths):
         to_device = self.gui.stack.currentIndex() != 0
         self._add_books(paths, to_device)
 
-    def files_dropped_on_book(self, event, paths):
+    def remote_file_dropped_on_book(self, url, fname):
+        if self.gui.current_view() is not self.gui.library_view:
+            return
+        db = self.gui.library_view.model().db
+        current_idx = self.gui.library_view.currentIndex()
+        if not current_idx.isValid(): return
+        cid = db.id(current_idx.row())
+        from calibre.gui2.dnd import DownloadDialog
+        d = DownloadDialog(url, fname, self.gui)
+        d.start_download()
+        if d.err is None:
+            self.files_dropped_on_book(None, [d.fpath], cid=cid)
+
+    def files_dropped_on_book(self, event, paths, cid=None):
         accept = False
         if self.gui.current_view() is not self.gui.library_view:
             return
         db = self.gui.library_view.model().db
         cover_changed = False
         current_idx = self.gui.library_view.currentIndex()
-        if not current_idx.isValid(): return
-        cid = db.id(current_idx.row())
+        if cid is None:
+            if not current_idx.isValid(): return
+            cid = db.id(current_idx.row()) if cid is None else cid
         for path in paths:
             ext = os.path.splitext(path)[1].lower()
             if ext:
@@ -154,8 +241,9 @@ class AddAction(InterfaceAction):
             elif ext in BOOK_EXTENSIONS:
                 db.add_format_with_hooks(cid, ext, path, index_is_id=True)
                 accept = True
-        if accept:
+        if accept and event is not None:
             event.accept()
+        if current_idx.isValid():
             self.gui.library_view.model().current_changed(current_idx, current_idx)
         if cover_changed:
             if self.gui.cover_flow:
@@ -188,26 +276,14 @@ class AddAction(InterfaceAction):
         '''
         Add books from the local filesystem to either the library or the device.
         '''
-        filters = [
-                        (_('Books'), BOOK_EXTENSIONS),
-                        (_('EPUB Books'), ['epub']),
-                        (_('LRF Books'), ['lrf']),
-                        (_('HTML Books'), ['htm', 'html', 'xhtm', 'xhtml']),
-                        (_('LIT Books'), ['lit']),
-                        (_('MOBI Books'), ['mobi', 'prc', 'azw']),
-                        (_('Topaz books'), ['tpz','azw1']),
-                        (_('Text books'), ['txt', 'rtf']),
-                        (_('PDF Books'), ['pdf']),
-                        (_('SNB Books'), ['snb']),
-                        (_('Comics'), ['cbz', 'cbr', 'cbc']),
-                        (_('Archives'), ['zip', 'rar']),
-                        ]
+        filters = get_filters()
         to_device = self.gui.stack.currentIndex() != 0
         if to_device:
-            filters = [(_('Supported books'), self.gui.device_manager.device.FORMATS)]
+            fmts = self.gui.device_manager.device.settings().format_map
+            filters = [(_('Supported books'), fmts)]
 
-        books = choose_files(self.gui, 'add books dialog dir', 'Select books',
-                             filters=filters)
+        books = choose_files(self.gui, 'add books dialog dir',
+                _('Select books'), filters=filters)
         if not books:
             return
         self._add_books(books, to_device)
@@ -239,13 +315,25 @@ class AddAction(InterfaceAction):
             if hasattr(self.gui, 'db_images'):
                 self.gui.db_images.reset()
             self.gui.tags_view.recount()
+
         if getattr(self._adder, 'merged_books', False):
             books = u'\n'.join([x if isinstance(x, unicode) else
                     x.decode(preferred_encoding, 'replace') for x in
                     self._adder.merged_books])
             info_dialog(self.gui, _('Merged some books'),
-                    _('Some duplicates were found and merged into the '
-                        'following existing books:'), det_msg=books, show=True)
+                    _('The following duplicate books were found and incoming book formats were '
+                        'processed and merged into your Calibre database according to your automerge settings:'), det_msg=books, show=True)
+
+        if getattr(self._adder, 'number_of_books_added', 0) > 0 or \
+                getattr(self._adder, 'merged_books', False):
+            # The formats of the current book could have changed if
+            # automerge is enabled
+            current_idx = self.gui.library_view.currentIndex()
+            if current_idx.isValid():
+                self.gui.library_view.model().current_changed(current_idx,
+                        current_idx)
+
+
         if getattr(self._adder, 'critical', None):
             det_msg = []
             for name, log in self._adder.critical.items():
