@@ -140,6 +140,8 @@ class DeviceManager(Thread): # {{{
         self.mount_connection_requests = Queue.Queue(0)
         self.open_feedback_slot = open_feedback_slot
         self.open_feedback_msg = open_feedback_msg
+        self._device_information = None
+        self.current_library_uuid = None
 
     def report_progress(self, *args):
         pass
@@ -159,7 +161,7 @@ class DeviceManager(Thread): # {{{
             try:
                 dev.reset(detected_device=detected_device,
                     report_progress=self.report_progress)
-                dev.open()
+                dev.open(self.current_library_uuid)
             except OpenFeedback, e:
                 if dev not in self.ejected_devices:
                     self.open_feedback_msg(dev.get_gui_name(), e.feedback_msg)
@@ -194,6 +196,7 @@ class DeviceManager(Thread): # {{{
         else:
             self.connected_slot(False, self.connected_device_kind)
         self.connected_device = None
+        self._device_information = None
 
     def detect_device(self):
         self.scanner.scan()
@@ -292,9 +295,13 @@ class DeviceManager(Thread): # {{{
 
     def _get_device_information(self):
         info = self.device.get_device_information(end_session=False)
-        info = [i.replace('\x00', '').replace('\x01', '') for i in info]
+        if len(info) < 5:
+            info = tuple(list(info) + [{}])
+        info = [i.replace('\x00', '').replace('\x01', '') if isinstance(i, basestring) else i
+                 for i in info]
         cp = self.device.card_prefix(end_session=False)
         fs = self.device.free_space()
+        self._device_information = {'info': info, 'prefixes': cp, 'freespace': fs}
         return info, cp, fs
 
     def get_device_information(self, done):
@@ -302,6 +309,9 @@ class DeviceManager(Thread): # {{{
         return self.create_job(self._get_device_information, done,
                     description=_('Get device information'))
 
+    def get_current_device_information(self):
+        return self._device_information
+
     def _books(self):
         '''Get metadata from device'''
         mainlist = self.device.books(oncard=None, end_session=False)
@@ -417,6 +427,13 @@ class DeviceManager(Thread): # {{{
         return self.create_job(self._view_book, done, args=[path, target],
                         description=_('View book on device'))
 
+    def set_current_library_uuid(self, uuid):
+        self.current_library_uuid = uuid
+
+    def set_driveinfo_name(self, location_code, name):
+        if self.connected_device:
+            self.connected_device.set_driveinfo_name(location_code, name)
+
     # }}}
 
 class DeviceAction(QAction): # {{{
@@ -1026,6 +1043,22 @@ class DeviceMixin(object): # {{{
                     self.location_manager.free[1] : 'carda',
                     self.location_manager.free[2] : 'cardb' }
                 on_card = space.get(sorted(space.keys(), reverse=True)[0], None)
+                try:
+                    total_size = sum([os.stat(f).st_size for f in files])
+                except:
+                    try:
+                        import traceback
+                        traceback.print_exc()
+                    except:
+                        pass
+                    total_size = self.location_manager.free[0]
+                loc = tweaks['send_news_to_device_location']
+                loc_index = {"carda": 1, "cardb": 2}.get(loc, 0)
+                if self.location_manager.free[loc_index] > total_size + (1024**2):
+                    # Send news to main memory if enough space available
+                    # as some devices like the Nook Color cannot handle
+                    # periodicals on SD cards properly
+                    on_card = loc if loc in ('carda', 'cardb') else None
                 self.upload_books(files, names, metadata,
                         on_card=on_card,
                         memory=[files, remove])
@@ -1129,6 +1162,14 @@ class DeviceMixin(object): # {{{
                 ), bad)
             d.exec_()
 
+    def upload_dirtied_booklists(self):
+        '''
+        Upload metadata to device.
+        '''
+        plugboards = self.library_view.model().db.prefs.get('plugboards', {})
+        self.device_manager.sync_booklists(Dispatcher(lambda x: x),
+                                           self.booklists(), plugboards)
+
     def upload_booklists(self):
         '''
         Upload metadata to device.
@@ -1278,6 +1319,16 @@ class DeviceMixin(object): # {{{
         to both speed up matching and to count matches.
         '''
 
+        if not self.device_manager.is_device_connected:
+            return False
+
+        # It might be possible to get here without having initialized the
+        # library view. In this case, simply give up
+        try:
+            db = self.library_view.model().db
+        except:
+            return False
+
         string_pat = re.compile('(?u)\W|[_]')
         def clean_string(x):
             x = x.lower() if x else ''
@@ -1285,26 +1336,19 @@ class DeviceMixin(object): # {{{
 
         update_metadata = prefs['manage_device_metadata'] == 'on_connect'
 
+        get_covers = False
+        if update_metadata and self.device_manager.is_device_connected:
+            if self.device_manager.device.WANTS_UPDATED_THUMBNAILS:
+                get_covers = True
+
         # Force a reset if the caches are not initialized
         if reset or not hasattr(self, 'db_book_title_cache'):
             # Build a cache (map) of the library, so the search isn't On**2
             db_book_title_cache = {}
             db_book_uuid_cache = {}
-            # It might be possible to get here without having initialized the
-            # library view. In this case, simply give up
-            try:
-                db = self.library_view.model().db
-            except:
-                return False
 
-            get_covers = False
-            if update_metadata and self.device_manager.is_device_connected:
-                if self.device_manager.device.WANTS_UPDATED_THUMBNAILS:
-                    get_covers = True
-
-            for id in db.data.iterallids():
-                mi = db.get_metadata(id, index_is_id=True, get_cover=get_covers)
-                title = clean_string(mi.title)
+            for id_ in db.data.iterallids():
+                title = clean_string(db.title(id_, index_is_id=True))
                 if title not in db_book_title_cache:
                     db_book_title_cache[title] = \
                                 {'authors':{}, 'author_sort':{}, 'db_ids':{}}
@@ -1312,14 +1356,14 @@ class DeviceMixin(object): # {{{
                 # and author, then remember the last one. That is OK, because as
                 # we can't tell the difference between the books, one is as good
                 # as another.
-                if mi.authors:
-                    authors = clean_string(authors_to_string(mi.authors))
-                    db_book_title_cache[title]['authors'][authors] = mi
-                if mi.author_sort:
-                    aus = clean_string(mi.author_sort)
-                    db_book_title_cache[title]['author_sort'][aus] = mi
-                db_book_title_cache[title]['db_ids'][mi.application_id] = mi
-                db_book_uuid_cache[mi.uuid] = mi
+                authors = clean_string(db.authors(id_, index_is_id=True))
+                if authors:
+                    db_book_title_cache[title]['authors'][authors] = id_
+                if db.author_sort(id_, index_is_id=True):
+                    aus = clean_string(db.author_sort(id_, index_is_id=True))
+                    db_book_title_cache[title]['author_sort'][aus] = id_
+                db_book_title_cache[title]['db_ids'][id_] = id_
+                db_book_uuid_cache[db.uuid(id_, index_is_id=True)] = id_
             self.db_book_title_cache = db_book_title_cache
             self.db_book_uuid_cache = db_book_uuid_cache
 
@@ -1327,19 +1371,22 @@ class DeviceMixin(object): # {{{
         # in_library field. If the UUID matches a book in the library, then
         # do not consider that book for other matching. In all cases set
         # the application_id to the db_id of the matching book. This value
-        # will be used by books_on_device to indicate matches.
+        # will be used by books_on_device to indicate matches. While we are
+        # going by, update the metadata for a book if automatic management is on
 
         for booklist in booklists:
             for book in booklist:
                 book.in_library = None
                 if getattr(book, 'uuid', None) in self.db_book_uuid_cache:
+                    id_ = db_book_uuid_cache[book.uuid]
                     if update_metadata:
-                        book.smart_update(self.db_book_uuid_cache[book.uuid],
+                        book.smart_update(db.get_metadata(id_,
+                                                          index_is_id=True,
+                                                          get_cover=get_covers),
                                           replace_metadata=True)
                     book.in_library = 'UUID'
                     # ensure that the correct application_id is set
-                    book.application_id = \
-                        self.db_book_uuid_cache[book.uuid].application_id
+                    book.application_id = id_
                     continue
                 # No UUID exact match. Try metadata matching.
                 book_title = clean_string(book.title)
@@ -1349,21 +1396,25 @@ class DeviceMixin(object): # {{{
                     # will match if any of the db_id, author, or author_sort
                     # also match.
                     if getattr(book, 'application_id', None) in d['db_ids']:
-                        # app_id already matches a db_id. No need to set it.
                         if update_metadata:
-                            book.smart_update(d['db_ids'][book.application_id],
+                            id_ = getattr(book, 'application_id', None)
+                            book.smart_update(db.get_metadata(id_,
+                                                              index_is_id=True,
+                                                              get_cover=get_covers),
                                               replace_metadata=True)
                         book.in_library = 'APP_ID'
+                        # app_id already matches a db_id. No need to set it.
                         continue
                     # Sonys know their db_id independent of the application_id
                     # in the metadata cache. Check that as well.
                     if getattr(book, 'db_id', None) in d['db_ids']:
                         if update_metadata:
-                            book.smart_update(d['db_ids'][book.db_id],
+                            book.smart_update(db.get_metadata(book.db_id,
+                                                              index_is_id=True,
+                                                              get_cover=get_covers),
                                               replace_metadata=True)
                         book.in_library = 'DB_ID'
-                        book.application_id = \
-                                    d['db_ids'][book.db_id].application_id
+                        book.application_id = book.db_id
                         continue
                     # We now know that the application_id is not right. Set it
                     # to None to prevent book_on_device from accidentally
@@ -1375,19 +1426,23 @@ class DeviceMixin(object): # {{{
                         # either can appear as the author
                         book_authors = clean_string(authors_to_string(book.authors))
                         if book_authors in d['authors']:
+                            id_ = d['authors'][book_authors]
                             if update_metadata:
-                                book.smart_update(d['authors'][book_authors],
-                                                  replace_metadata=True)
+                                book.smart_update(db.get_metadata(id_,
+                                                              index_is_id=True,
+                                                              get_cover=get_covers),
+                                              replace_metadata=True)
                             book.in_library = 'AUTHOR'
-                            book.application_id = \
-                                    d['authors'][book_authors].application_id
+                            book.application_id = id_
                         elif book_authors in d['author_sort']:
+                            id_ = d['author_sort'][book_authors]
                             if update_metadata:
-                                book.smart_update(d['author_sort'][book_authors],
+                                book.smart_update(db.get_metadata(id_,
+                                                              index_is_id=True,
+                                                              get_cover=get_covers),
                                                   replace_metadata=True)
                             book.in_library = 'AUTH_SORT'
-                            book.application_id = \
-                                d['author_sort'][book_authors].application_id
+                            book.application_id = id_
                 else:
                     # Book definitely not matched. Clear its application ID
                     book.application_id = None
