@@ -3,11 +3,11 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
 '''Dialog to edit metadata in bulk'''
 
-import re, os
+import re, os, inspect
 
 from PyQt4.Qt import Qt, QDialog, QGridLayout, QVBoxLayout, QFont, QLabel, \
                      pyqtSignal, QDialogButtonBox, QInputDialog, QLineEdit, \
-                     QDate
+                     QDate, QCompleter
 
 from calibre.gui2.dialogs.metadata_bulk_ui import Ui_MetadataBulkDialog
 from calibre.gui2.dialogs.tag_editor import TagEditor
@@ -364,7 +364,8 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
                     (fm[f]['datatype'] in ['text', 'series', 'enumeration']
                      and fm[f].get('search_terms', None)
                      and f not in ['formats', 'ondevice']) or
-                    fm[f]['datatype'] in ['int', 'float', 'bool'] ):
+                    (fm[f]['datatype'] in ['int', 'float', 'bool'] and
+                     f not in ['id'])):
                 self.all_fields.append(f)
                 self.writable_fields.append(f)
             if fm[f]['datatype'] == 'composite':
@@ -393,6 +394,14 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
             self.book_1_text.setObjectName(name)
             self.testgrid.addWidget(w, i+offset, 2, 1, 1)
 
+        ident_types = sorted(self.db.get_all_identifier_types(), key=sort_key)
+        self.s_r_dst_ident.setCompleter(QCompleter(ident_types))
+        try:
+            self.s_r_dst_ident.setPlaceholderText(_('Enter an identifier type'))
+        except:
+            pass
+        self.s_r_src_ident.addItems(ident_types)
+
         self.main_heading = _(
                  '<b>You can destroy your library using this feature.</b> '
                  'Changes are permanent. There is no undo function. '
@@ -449,6 +458,8 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
         self.test_text.editTextChanged[str].connect(self.s_r_paint_results)
         self.comma_separated.stateChanged.connect(self.s_r_paint_results)
         self.case_sensitive.stateChanged.connect(self.s_r_paint_results)
+        self.s_r_src_ident.currentIndexChanged[int].connect(self.s_r_paint_results)
+        self.s_r_dst_ident.textChanged.connect(self.s_r_paint_results)
         self.s_r_template.lost_focus.connect(self.s_r_template_changed)
         self.central_widget.setCurrentIndex(0)
 
@@ -471,6 +482,8 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
         self.query_field.addItems(sorted([q for q in self.queries], key=sort_key))
         self.query_field.currentIndexChanged[str].connect(self.s_r_query_change)
         self.query_field.setCurrentIndex(0)
+        self.search_field.setCurrentIndex(0)
+        self.s_r_search_field_changed(0)
 
     def s_r_sf_itemdata(self, idx):
         if idx is None:
@@ -495,6 +508,13 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
                 val = mi.get(field, None)
             if isinstance(val, (int, float, bool)):
                 val = str(val)
+            elif fm['is_csp']:
+                # convert the csp dict into a list
+                id_type = unicode(self.s_r_src_ident.currentText())
+                if id_type:
+                    val = [val.get(id_type, '')]
+                else:
+                    val = [u'%s:%s'%(t[0], t[1]) for t in val.iteritems()]
             if val is None:
                 val = [] if fm['is_multiple'] else ['']
             elif not fm['is_multiple']:
@@ -512,12 +532,17 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
         self.s_r_search_field_changed(self.search_field.currentIndex())
 
     def s_r_search_field_changed(self, idx):
-        if self.search_mode.currentIndex() != 0 and idx == 1: # Template
+        self.s_r_template.setVisible(False)
+        self.template_label.setVisible(False)
+        self.s_r_src_ident_label.setVisible(False)
+        self.s_r_src_ident.setVisible(False)
+        if idx == 1: # Template
             self.s_r_template.setVisible(True)
             self.template_label.setVisible(True)
-        else:
-            self.s_r_template.setVisible(False)
-            self.template_label.setVisible(False)
+        elif self.s_r_sf_itemdata(idx) == 'identifiers':
+            self.s_r_src_ident_label.setVisible(True)
+            self.s_r_src_ident.setVisible(True)
+
         for i in range(0, self.s_r_number_of_books):
             w = getattr(self, 'book_%d_text'%(i+1))
             mi = self.db.get_metadata(self.ids[i], index_is_id=True)
@@ -535,10 +560,15 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
             self.s_r_paint_results(None)
 
     def s_r_destination_field_changed(self, idx):
+        self.s_r_dst_ident_label.setVisible(False)
+        self.s_r_dst_ident.setVisible(False)
         txt = self.s_r_df_itemdata(idx)
         if not txt:
             txt = self.s_r_sf_itemdata(None)
         if txt and txt in self.writable_fields:
+            if txt == 'identifiers':
+                self.s_r_dst_ident_label.setVisible(True)
+                self.s_r_dst_ident.setVisible(True)
             self.destination_field_fm = self.db.metadata_for_field(txt)
         self.s_r_paint_results(None)
 
@@ -617,6 +647,10 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
             dest = src
         dest_mode = self.replace_mode.currentIndex()
 
+        if self.destination_field_fm['is_csp']:
+            if not unicode(self.s_r_dst_ident.text()):
+                raise Exception(_('You must specify a destination identifier type'))
+
         if self.destination_field_fm['is_multiple']:
             if self.comma_separated.isChecked():
                 if dest == 'authors':
@@ -635,6 +669,13 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
 
         if dest_mode != 0:
             dest_val = mi.get(dest, '')
+            if self.db.metadata_for_field(dest)['is_csp']:
+                dst_id_type = unicode(self.s_r_dst_ident.text())
+                if dst_id_type:
+                    dest_val = [dest_val.get(dst_id_type, '')]
+                else:
+                    # convert the csp dict into a list
+                    dest_val = [u'%s:%s'%(t[0], t[1]) for t in dest_val.iteritems()]
             if dest_val is None:
                 dest_val = []
             elif not isinstance(dest_val, list):
@@ -711,16 +752,23 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
         val = self.s_r_do_regexp(mi)
         val = self.s_r_do_destination(mi, val)
         if dfm['is_multiple']:
-            if dfm['is_custom']:
-                # The standard tags and authors values want to be lists.
-                # All custom columns are to be strings
-                val = dfm['is_multiple'].join(val)
             if dest == 'authors' and len(val) == 0:
                 error_dialog(self, _('Search/replace invalid'),
                              _('Authors cannot be set to the empty string. '
                                'Book title %s not processed')%mi.title,
                              show=True)
                 return
+            # convert the colon-separated pair strings back into a dict, which
+            # is what set_identifiers wants
+            if dfm['is_csp']:
+                dst_id_type = unicode(self.s_r_dst_ident.text())
+                if dst_id_type:
+                    v = ''.join(val)
+                    ids = mi.get(dest)
+                    ids[dst_id_type] = v
+                    val = ids
+                else:
+                    val = dict([(t.split(':')) for t in val])
         else:
             val = self.s_r_replace_mode_separator().join(val)
             if dest == 'title' and len(val) == 0:
@@ -732,8 +780,15 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
 
         if dfm['is_custom']:
             extra = self.db.get_custom_extra(id, label=dfm['label'], index_is_id=True)
-            self.db.set_custom(id, val, label=dfm['label'], extra=extra,
-                               commit=False)
+            books_to_refresh = self.db.set_custom(id, val, label=dfm['label'],
+                                                  extra=extra, commit=False,
+                                                  allow_case_change=True)
+        elif dest.startswith('#') and dest.endswith('_index'):
+            label = self.db.field_metadata[dest[:-6]]['label']
+            series = self.db.get_custom(id, label=label, index_is_id=True)
+            books_to_refresh = self.db.set_custom(id, series, label=label,
+                                                  extra=val, commit=False,
+                                                  allow_case_change=True)
         else:
             if dest == 'comments':
                 setter = self.db.set_comment
@@ -741,10 +796,19 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
                 setter = self.db.set_title_sort
             else:
                 setter = getattr(self.db, 'set_'+dest)
-            if dest in ['title', 'authors']:
-                setter(id, val, notify=False)
+
+            args = inspect.getargspec(setter)
+            if args and 'allow_case_change' in args.args:
+                books_to_refresh = setter(id, val, notify=False, commit=False,
+                                          allow_case_change=True)
             else:
                 setter(id, val, notify=False, commit=False)
+                books_to_refresh = set([])
+        if books_to_refresh:
+            # Must commit here because we are telling the gui that the data is
+            # permanent. Make sure it really is.
+            self.db.commit()
+            self.model.refresh_ids(list(books_to_refresh))
 
     def create_custom_column_editors(self):
         w = self.central_widget.widget(1)
@@ -781,6 +845,7 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
 
         self.authors.set_separator('&')
         self.authors.set_space_before_sep(True)
+        self.authors.set_add_separator(tweaks['authors_completer_append_separator'])
         self.authors.update_items_cache(self.db.all_author_names())
 
     def initialize_series(self):
@@ -911,6 +976,7 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
 
     def series_changed(self, *args):
         self.write_series = True
+        self.autonumber_series.setEnabled(True)
 
     def s_r_remove_query(self, *args):
         if self.query_field.currentIndex() == 0:
@@ -953,11 +1019,13 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
         query['search_field'] = unicode(self.search_field.currentText())
         query['search_mode'] = unicode(self.search_mode.currentText())
         query['s_r_template'] = unicode(self.s_r_template.text())
+        query['s_r_src_ident'] = unicode(self.s_r_src_ident.currentText())
         query['search_for'] = unicode(self.search_for.text())
         query['case_sensitive'] = self.case_sensitive.isChecked()
         query['replace_with'] = unicode(self.replace_with.text())
         query['replace_func'] = unicode(self.replace_func.currentText())
         query['destination_field'] = unicode(self.destination_field.currentText())
+        query['s_r_dst_ident'] = unicode(self.s_r_dst_ident.text())
         query['replace_mode'] = unicode(self.replace_mode.currentText())
         query['comma_separated'] = self.comma_separated.isChecked()
         query['results_count'] = self.results_count.value()
@@ -984,37 +1052,61 @@ class MetadataBulkDialog(ResizableDialog, Ui_MetadataBulkDialog):
             self.s_r_reset_query_fields()
             return
 
-        def set_index(attr, txt):
+        def set_text(attr, key):
             try:
-                attr.setCurrentIndex(attr.findText(txt))
+                attr.setText(item[key])
+            except:
+                pass
+
+        def set_checked(attr, key):
+            try:
+                attr.setChecked(item[key])
+            except:
+                attr.setChecked(False)
+
+        def set_value(attr, key):
+            try:
+                attr.setValue(int(item[key]))
+            except:
+                attr.setValue(0)
+
+        def set_index(attr, key):
+            try:
+                attr.setCurrentIndex(attr.findText(item[key]))
             except:
                 attr.setCurrentIndex(0)
 
-        set_index(self.search_mode, item['search_mode'])
-        set_index(self.search_field, item['search_field'])
-        self.s_r_template.setText(item['s_r_template'])
+        set_index(self.search_mode, 'search_mode')
+        set_index(self.search_field, 'search_field')
+        set_text(self.s_r_template, 's_r_template')
+
         self.s_r_template_changed() #simulate gain/loss of focus
-        self.search_for.setText(item['search_for'])
-        self.case_sensitive.setChecked(item['case_sensitive'])
-        self.replace_with.setText(item['replace_with'])
-        set_index(self.replace_func, item['replace_func'])
-        set_index(self.destination_field, item['destination_field'])
-        set_index(self.replace_mode, item['replace_mode'])
-        self.comma_separated.setChecked(item['comma_separated'])
-        self.results_count.setValue(int(item['results_count']))
-        self.starting_from.setValue(int(item['starting_from']))
-        self.multiple_separator.setText(item['multiple_separator'])
+
+        set_index(self.s_r_src_ident, 's_r_src_ident');
+        set_text(self.s_r_dst_ident, 's_r_dst_ident')
+        set_text(self.search_for, 'search_for')
+        set_checked(self.case_sensitive, 'case_sensitive')
+        set_text(self.replace_with, 'replace_with')
+        set_index(self.replace_func, 'replace_func')
+        set_index(self.destination_field, 'destination_field')
+        set_index(self.replace_mode, 'replace_mode')
+        set_checked(self.comma_separated, 'comma_separated')
+        set_value(self.results_count, 'results_count')
+        set_value(self.starting_from, 'starting_from')
+        set_text(self.multiple_separator, 'multiple_separator')
 
     def s_r_reset_query_fields(self):
         # Don't reset the search mode. The user will probably want to use it
         # as it was
         self.search_field.setCurrentIndex(0)
+        self.s_r_src_ident.setCurrentIndex(0)
         self.s_r_template.setText("")
         self.search_for.setText("")
         self.case_sensitive.setChecked(False)
         self.replace_with.setText("")
         self.replace_func.setCurrentIndex(0)
         self.destination_field.setCurrentIndex(0)
+        self.s_r_dst_ident.setText('')
         self.replace_mode.setCurrentIndex(0)
         self.comma_separated.setChecked(True)
         self.results_count.setValue(999)
