@@ -12,7 +12,7 @@ from threading import Thread
 
 from PyQt4.Qt import SIGNAL, QObject, Qt, QTimer, QDate, \
     QPixmap, QListWidgetItem, QDialog, pyqtSignal, QIcon, \
-    QPushButton
+    QPushButton, QKeySequence
 
 from calibre.gui2 import error_dialog, file_icon_provider, dynamic, \
                            choose_files, choose_images, ResizableDialog, \
@@ -472,17 +472,19 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             self.prev_button = QPushButton(QIcon(I('back.png')), _('Previous'),
                     self)
             self.button_box.addButton(self.prev_button, self.button_box.ActionRole)
-            tip = _('Save changes and edit the metadata of %s')%prev
+            tip = (_('Save changes and edit the metadata of %s')+' [Alt+Left]')%prev
             self.prev_button.setToolTip(tip)
             self.prev_button.clicked.connect(partial(self.next_triggered,
                 -1))
+            self.prev_button.setShortcut(QKeySequence('Alt+Left'))
         if next_:
             self.next_button = QPushButton(QIcon(I('forward.png')), _('Next'),
                     self)
             self.button_box.addButton(self.next_button, self.button_box.ActionRole)
-            tip = _('Save changes and edit the metadata of %s')%next_
+            tip = (_('Save changes and edit the metadata of %s')+' [Alt+Right]')%next_
             self.next_button.setToolTip(tip)
             self.next_button.clicked.connect(partial(self.next_triggered, 1))
+            self.next_button.setShortcut(QKeySequence('Alt+Right'))
 
         self.splitter.setStretchFactor(100, 1)
         self.read_state()
@@ -616,6 +618,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         self.original_series_name = unicode(self.series.text()).strip()
         if len(db.custom_column_label_map) == 0:
             self.central_widget.tabBar().setVisible(False)
+            self.central_widget.setTabEnabled(1, False)
         else:
             self.create_custom_column_editors()
         self.generate_cover_button.clicked.connect(self.generate_cover)
@@ -734,6 +737,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
 
         self.authors.set_separator('&')
         self.authors.set_space_before_sep(True)
+        self.authors.set_add_separator(tweaks['authors_completer_append_separator'])
         self.authors.update_items_cache(self.db.all_author_names())
 
     def initialize_series(self):
@@ -780,8 +784,8 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                     _('You have changed the tags. In order to use the tags'
                        ' editor, you must either discard or apply these '
                        'changes. Apply changes?'), show_copy_button=False):
-                self.books_to_refresh |= self.apply_tags(commit=True, notify=True,
-                                                         allow_case_change=True)
+                self.books_to_refresh |= self.apply_tags(commit=True,
+                        notify=True)
                 self.original_tags = unicode(self.tags.text())
             else:
                 self.tags.setText(self.original_tags)
@@ -951,8 +955,8 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             for w in getattr(self, 'custom_column_widgets', []):
                 self.books_to_refresh |= w.commit(self.id)
             self.db.commit()
-        except IOError, err:
-            if err.errno == 13: # Permission denied
+        except (IOError, OSError) as err:
+            if getattr(err, 'errno', -1) == 13: # Permission denied
                 fname = err.filename if err.filename else 'file'
                 return error_dialog(self, _('Permission denied'),
                         _('Could not open %s. Is it being used by another'
