@@ -8,38 +8,201 @@ Scheduler for automated recipe downloads
 '''
 
 from datetime import timedelta
+import calendar, textwrap
 
-from PyQt4.Qt import QDialog, SIGNAL, Qt, QTime, QObject, QMenu, \
-        QAction, QIcon, QMutex, QTimer, pyqtSignal, QWidget, QHBoxLayout, \
-        QLabel
+from PyQt4.Qt import QDialog, Qt, QTime, QObject, QMenu, QHBoxLayout, \
+        QAction, QIcon, QMutex, QTimer, pyqtSignal, QWidget, QGridLayout, \
+        QCheckBox, QTimeEdit, QLabel, QLineEdit, QDoubleSpinBox
 
 from calibre.gui2.dialogs.scheduler_ui import Ui_Dialog
-from calibre.gui2.search_box import SearchBox2
 from calibre.gui2 import config as gconf, error_dialog
 from calibre.web.feeds.recipes.model import RecipeModel
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.date import utcnow
 from calibre.utils.network import internet_connected
+from calibre.utils.ordered_dict import OrderedDict
+from calibre import force_unicode
+
+def convert_day_time_schedule(val):
+    day_of_week, hour, minute = val
+    if day_of_week == -1:
+        return (tuple(xrange(7)), hour, minute)
+    return ((day_of_week,), hour, minute)
+
+class Base(QWidget):
+
+    def __init__(self, parent=None):
+        QWidget.__init__(self, parent)
+        self.l = QGridLayout()
+        self.setLayout(self.l)
+        self.setToolTip(textwrap.dedent(self.HELP))
+
+class DaysOfWeek(Base):
+
+    HELP = _('''\
+                Download this periodical every week on the specified days after
+                the specified time. For example, if you choose: Monday after
+                9:00 AM, then the periodical will be download every Monday as
+                soon after 9:00 AM as possible.
+            ''')
+
+    def __init__(self, parent=None):
+        Base.__init__(self, parent)
+        self.days = [QCheckBox(force_unicode(calendar.day_abbr[d]),
+            self) for d in xrange(7)]
+        for i, cb in enumerate(self.days):
+            row = i % 2
+            col = i // 2
+            self.l.addWidget(cb, row, col, 1, 1)
+
+        self.time = QTimeEdit(self)
+        self.time.setDisplayFormat('hh:mm AP')
+        self.hl = QHBoxLayout()
+        self.l1 = QLabel(_('&Download after:'))
+        self.l1.setBuddy(self.time)
+        self.hl.addWidget(self.l1)
+        self.hl.addWidget(self.time)
+        self.l.addLayout(self.hl, 1, 3, 1, 1)
+        self.initialize()
+
+    def initialize(self, typ=None, val=None):
+        if typ is None:
+            typ = 'day/time'
+            val = (-1, 9, 0)
+        if typ == 'day/time':
+            val = convert_day_time_schedule(val)
+
+        days_of_week, hour, minute = val
+        for i, d in enumerate(self.days):
+            d.setChecked(i in days_of_week)
+
+        self.time.setTime(QTime(hour, minute))
+
+    @property
+    def schedule(self):
+        days_of_week = tuple([i for i, d in enumerate(self.days) if
+            d.isChecked()])
+        t = self.time.time()
+        hour, minute = t.hour(), t.minute()
+        return 'days_of_week', (days_of_week, int(hour), int(minute))
+
+class DaysOfMonth(Base):
+
+    HELP = _('''\
+                Download this periodical every month, on the specified days.
+                The download will happen as soon after the specified time as
+                possible on the specified days of each month. For example,
+                if you choose the 1st and the 15th after 9:00 AM, the
+                periodical will be downloaded on the 1st and 15th of every
+                month, as soon after 9:00 AM as possible.
+            ''')
+
+    def __init__(self, parent=None):
+        Base.__init__(self, parent)
+
+        self.l1 = QLabel(_('&Days of the month:'))
+        self.days = QLineEdit(self)
+        self.days.setToolTip(_('Comma separated list of days of the month.'
+            ' For example: 1, 15'))
+        self.l1.setBuddy(self.days)
+
+        self.l2 = QLabel(_('Download &after:'))
+        self.time = QTimeEdit(self)
+        self.time.setDisplayFormat('hh:mm AP')
+        self.l2.setBuddy(self.time)
+
+        self.l.addWidget(self.l1, 0, 0, 1, 1)
+        self.l.addWidget(self.days, 0, 1, 1, 1)
+        self.l.addWidget(self.l2, 1, 0, 1, 1)
+        self.l.addWidget(self.time, 1, 1, 1, 1)
+
+    def initialize(self, typ=None, val=None):
+        if val is None:
+            val = ((1,), 9, 0)
+        days_of_month, hour, minute = val
+        self.days.setText(', '.join(map(str, map(int, days_of_month))))
+        self.time.setTime(QTime(hour, minute))
+
+    @property
+    def schedule(self):
+        parts = [x.strip() for x in unicode(self.days.text()).split(',') if
+                x.strip()]
+        try:
+            days_of_month = tuple(map(int, parts))
+        except:
+            days_of_month = (1,)
+        if not days_of_month:
+            days_of_month = (1,)
+        t = self.time.time()
+        hour, minute = t.hour(), t.minute()
+        return 'days_of_month', (days_of_month, int(hour), int(minute))
+
+class EveryXDays(Base):
+
+    HELP = _('''\
+                Download this periodical every x days. For example, if you
+                choose 30 days, the periodical will be downloaded every 30
+                days. Note that you can set periods of less than a day, like
+                0.1 days to download a periodical more than once a day.
+            ''')
+
+    def __init__(self, parent=None):
+        Base.__init__(self, parent)
+        self.l1 = QLabel(_('&Download every:'))
+        self.interval = QDoubleSpinBox(self)
+        self.interval.setMinimum(0.04)
+        self.interval.setSpecialValueText(_('every hour'))
+        self.interval.setMaximum(1000.0)
+        self.interval.setValue(31.0)
+        self.interval.setSuffix(' ' + _('days'))
+        self.interval.setSingleStep(1.0)
+        self.interval.setDecimals(2)
+        self.l1.setBuddy(self.interval)
+        self.l2 = QLabel(_('Note: You can set intervals of less than a day,'
+            ' by typing the value manually.'))
+        self.l2.setWordWrap(True)
+
+        self.l.addWidget(self.l1, 0, 0, 1, 1)
+        self.l.addWidget(self.interval, 0, 1, 1, 1)
+        self.l.addWidget(self.l2, 1, 0, 1, -1)
+
+    def initialize(self, typ=None, val=None):
+        if val is None:
+            val = 31.0
+        self.interval.setValue(val)
+
+    @property
+    def schedule(self):
+        schedule = self.interval.value()
+        return 'interval', schedule
+
 
 class SchedulerDialog(QDialog, Ui_Dialog):
 
+    SCHEDULE_TYPES = OrderedDict([
+            ('days_of_week', DaysOfWeek),
+            ('days_of_month', DaysOfMonth),
+            ('every_x_days', EveryXDays),
+    ])
+
+    download = pyqtSignal(object)
+
     def __init__(self, recipe_model, parent=None):
         QDialog.__init__(self, parent)
         self.setupUi(self)
         self.recipe_model = recipe_model
         self.recipe_model.do_refresh()
+        self.count_label.setText(
+            _('%s news sources') %
+                self.recipe_model.showing_count)
+
+        self.schedule_widgets = []
+        for key in reversed(self.SCHEDULE_TYPES):
+            self.schedule_widgets.insert(0, self.SCHEDULE_TYPES[key](self))
+            self.schedule_stack.insertWidget(0, self.schedule_widgets[0])
 
-        self._cont = QWidget(self)
-        self._cont.l = QHBoxLayout()
-        self._cont.setLayout(self._cont.l)
-        self._cont.la = QLabel(_('&Search:'))
-        self._cont.l.addWidget(self._cont.la, 1)
-        self.search = SearchBox2(self)
-        self._cont.l.addWidget(self.search, 100)
-        self._cont.la.setBuddy(self.search)
-        self.search.setMinimumContentsLength(25)
         self.search.initialize('scheduler_search_history')
-        self.recipe_box.layout().insertWidget(0, self._cont)
+        self.search.setMinimumContentsLength(15)
         self.search.search.connect(self.recipe_model.search)
         self.recipe_model.searched.connect(self.search.search_done,
                 type=Qt.QueuedConnection)
@@ -51,29 +214,43 @@ class SchedulerDialog(QDialog, Ui_Dialog):
         self.detail_box.setVisible(False)
         self.download_button.setVisible(False)
         self.recipes.currentChanged = self.current_changed
-        self.interval_button.setChecked(True)
+        for b, c in self.SCHEDULE_TYPES.iteritems():
+            b = getattr(self, b)
+            b.toggled.connect(self.schedule_type_selected)
+            b.setToolTip(textwrap.dedent(c.HELP))
+        self.days_of_week.setChecked(True)
 
-        self.connect(self.schedule, SIGNAL('stateChanged(int)'),
-                self.toggle_schedule_info)
-        self.connect(self.show_password, SIGNAL('stateChanged(int)'),
-                     lambda state: self.password.setEchoMode(self.password.Normal if state == Qt.Checked else self.password.Password))
-        self.connect(self.download_button, SIGNAL('clicked()'),
-                self.download_clicked)
-        self.connect(self.download_all_button, SIGNAL('clicked()'),
+        self.schedule.stateChanged[int].connect(self.toggle_schedule_info)
+        self.show_password.stateChanged[int].connect(self.set_pw_echo_mode)
+        self.download_button.clicked.connect(self.download_clicked)
+        self.download_all_button.clicked.connect(
                 self.download_all_clicked)
 
         self.old_news.setValue(gconf['oldest_news'])
 
+    def set_pw_echo_mode(self, state):
+        self.password.setEchoMode(self.password.Normal
+                if state == Qt.Checked else self.password.Password)
+
+
+    def schedule_type_selected(self, *args):
+        for i, st in enumerate(self.SCHEDULE_TYPES):
+            if getattr(self, st).isChecked():
+                self.schedule_stack.setCurrentIndex(i)
+                break
+
     def keyPressEvent(self, ev):
         if ev.key() not in (Qt.Key_Enter, Qt.Key_Return):
             return QDialog.keyPressEvent(self, ev)
 
     def break_cycles(self):
-        self.disconnect(self.recipe_model,  SIGNAL('searched(PyQt_PyObject)'),
-                self.search_done)
-        self.disconnect(self.recipe_model,  SIGNAL('searched(PyQt_PyObject)'),
-                self.search.search_done)
-        self.search.search.disconnect()
+        try:
+            self.recipe_model.searched.disconnect(self.search_done)
+            self.recipe_model.searched.disconnect(self.search.search_done)
+            self.search.search.disconnect()
+            self.download.disconnect()
+        except:
+            pass
         self.recipe_model = None
 
     def search_done(self, *args):
@@ -82,8 +259,9 @@ class SchedulerDialog(QDialog, Ui_Dialog):
 
     def toggle_schedule_info(self, *args):
         enabled = self.schedule.isChecked()
-        for x in ('daily_button', 'day', 'time', 'interval_button', 'interval'):
+        for x in self.SCHEDULE_TYPES:
             getattr(self, x).setEnabled(enabled)
+        self.schedule_stack.setEnabled(enabled)
         self.last_downloaded.setVisible(enabled)
 
     def current_changed(self, current, previous):
@@ -105,14 +283,14 @@ class SchedulerDialog(QDialog, Ui_Dialog):
             return False
         return QDialog.accept(self)
 
-    def download_clicked(self):
+    def download_clicked(self, *args):
         self.commit()
         if self.commit() and self.current_urn:
-            self.emit(SIGNAL('download(PyQt_PyObject)'), self.current_urn)
+            self.download.emit(self.current_urn)
 
-    def download_all_clicked(self):
+    def download_all_clicked(self, *args):
         if self.commit() and self.commit():
-            self.emit(SIGNAL('download(PyQt_PyObject)'), None)
+            self.download.emit(None)
 
     @property
     def current_urn(self):
@@ -138,24 +316,19 @@ class SchedulerDialog(QDialog, Ui_Dialog):
                 self.recipe_model.set_account_info(urn, un, pw)
 
         if self.schedule.isChecked():
-            schedule_type = 'interval' if self.interval_button.isChecked() else 'day/time'
-            if schedule_type == 'interval':
-                schedule = self.interval.value()
-                if schedule < 0.1:
-                    schedule = 1./24.
-            else:
-                day_of_week = self.day.currentIndex() - 1
-                t = self.time.time()
-                hour, minute = t.hour(), t.minute()
-                schedule = (day_of_week, hour, minute)
+            schedule_type, schedule = \
+                    self.schedule_stack.currentWidget().schedule
             self.recipe_model.schedule_recipe(urn, schedule_type, schedule)
         else:
             self.recipe_model.un_schedule_recipe(urn)
 
         add_title_tag = self.add_title_tag.isChecked()
+        keep_issues = u'0'
+        if self.keep_issues.isEnabled():
+            keep_issues = unicode(self.keep_issues.value())
         custom_tags = unicode(self.custom_tags.text()).strip()
         custom_tags = [x.strip() for x in custom_tags.split(',')]
-        self.recipe_model.customize_recipe(urn, add_title_tag, custom_tags)
+        self.recipe_model.customize_recipe(urn, add_title_tag, custom_tags, keep_issues)
         return True
 
     def initialize_detail_box(self, urn):
@@ -197,27 +370,34 @@ class SchedulerDialog(QDialog, Ui_Dialog):
         self.schedule.setChecked(scheduled)
         self.toggle_schedule_info()
         self.last_downloaded.setText(_('Last downloaded: never'))
+        ld_text = _('never')
         if scheduled:
             typ, sch, last_downloaded = schedule_info
-            if typ == 'interval':
-                self.interval_button.setChecked(True)
-                self.interval.setValue(sch)
-            elif typ == 'day/time':
-                self.daily_button.setChecked(True)
-                day, hour, minute = sch
-                self.day.setCurrentIndex(day+1)
-                self.time.setTime(QTime(hour, minute))
-
             d = utcnow() - last_downloaded
             def hm(x): return (x-x%3600)//3600, (x%3600 - (x%3600)%60)//60
             hours, minutes = hm(d.seconds)
             tm = _('%d days, %d hours and %d minutes ago')%(d.days, hours, minutes)
             if d < timedelta(days=366):
-                self.last_downloaded.setText(_('Last downloaded')+': '+tm)
-
-        add_title_tag, custom_tags = customize_info
+                ld_text = tm
+        else:
+            typ, sch = 'day/time', (-1, 9, 0)
+        sch_widget = {'day/time': 0, 'days_of_week': 0, 'days_of_month':1,
+                'interval':2}[typ]
+        rb = getattr(self, list(self.SCHEDULE_TYPES)[sch_widget])
+        rb.setChecked(True)
+        self.schedule_stack.setCurrentIndex(sch_widget)
+        self.schedule_stack.currentWidget().initialize(typ, sch)
+        add_title_tag, custom_tags, keep_issues = customize_info
         self.add_title_tag.setChecked(add_title_tag)
         self.custom_tags.setText(u', '.join(custom_tags))
+        self.last_downloaded.setText(_('Last downloaded:') + ' ' + ld_text)
+        try:
+            keep_issues = int(keep_issues)
+        except:
+            keep_issues = 0
+        self.keep_issues.setValue(keep_issues)
+        self.keep_issues.setEnabled(self.add_title_tag.isChecked())
+
 
 
 class Scheduler(QObject):
@@ -231,7 +411,8 @@ class Scheduler(QObject):
         QObject.__init__(self, parent)
         self.internet_connection_failed = False
         self._parent = parent
-        self.recipe_model = RecipeModel(db)
+        self.recipe_model = RecipeModel()
+        self.db = db
         self.lock = QMutex(QMutex.Recursive)
         self.download_queue = set([])
 
@@ -239,9 +420,9 @@ class Scheduler(QObject):
         self.news_icon = QIcon(I('news.png'))
         self.scheduler_action = QAction(QIcon(I('scheduler.png')), _('Schedule news download'), self)
         self.news_menu.addAction(self.scheduler_action)
-        self.connect(self.scheduler_action, SIGNAL('triggered(bool)'), self.show_dialog)
+        self.scheduler_action.triggered[bool].connect(self.show_dialog)
         self.cac = QAction(QIcon(I('user_profile.png')), _('Add a custom news source'), self)
-        self.connect(self.cac, SIGNAL('triggered(bool)'), self.customize_feeds)
+        self.cac.triggered[bool].connect(self.customize_feeds)
         self.news_menu.addAction(self.cac)
         self.news_menu.addSeparator()
         self.all_action = self.news_menu.addAction(
@@ -250,22 +431,22 @@ class Scheduler(QObject):
 
         self.timer = QTimer(self)
         self.timer.start(int(self.INTERVAL * 60 * 1000))
-        self.connect(self.timer, SIGNAL('timeout()'), self.check)
+        self.timer.timeout.connect(self.check)
         self.oldest = gconf['oldest_news']
         QTimer.singleShot(5 * 1000, self.oldest_check)
-        self.database_changed = self.recipe_model.database_changed
+
+    def database_changed(self, db):
+        self.db = db
 
     def oldest_check(self):
         if self.oldest > 0:
             delta = timedelta(days=self.oldest)
             try:
-                ids = list(self.recipe_model.db.tags_older_than(_('News'),
+                ids = list(self.db.tags_older_than(_('News'),
                     delta))
             except:
-                # Should never happen
+                # Happens if library is being switched
                 ids = []
-                import traceback
-                traceback.print_exc()
             if ids:
                 if ids:
                     self.delete_old_news.emit(ids)
@@ -276,8 +457,7 @@ class Scheduler(QObject):
         self.lock.lock()
         try:
             d = SchedulerDialog(self.recipe_model)
-            self.connect(d, SIGNAL('download(PyQt_PyObject)'),
-                self.download_clicked)
+            d.download.connect(self.download_clicked)
             d.exec_()
             gconf['oldest_news'] = self.oldest = d.old_news.value()
             d.break_cycles()
@@ -299,7 +479,7 @@ class Scheduler(QObject):
             un = pw = None
             if account_info is not None:
                 un, pw = account_info
-            add_title_tag, custom_tags = customize_info
+            add_title_tag, custom_tags, keep_issues = customize_info
             script = self.recipe_model.get_recipe(urn)
             pt = PersistentTemporaryFile('_builtin.recipe')
             pt.write(script)
@@ -312,6 +492,7 @@ class Scheduler(QObject):
                     'recipe':pt.name,
                     'title':recipe.get('title',''),
                     'urn':urn,
+                    'keep_issues':keep_issues
                    }
             self.download_queue.add(urn)
             self.start_recipe_fetch.emit(arg)
@@ -371,7 +552,6 @@ class Scheduler(QObject):
 if __name__ == '__main__':
     from calibre.gui2 import is_ok_to_use_qt
     is_ok_to_use_qt()
-    from calibre.library.database2 import LibraryDatabase2
-    d = SchedulerDialog(RecipeModel(LibraryDatabase2('/home/kovid/documents/library')))
+    d = SchedulerDialog(RecipeModel())
     d.exec_()
 
