@@ -4,13 +4,13 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 import time, os
 
 from PyQt4.Qt import SIGNAL, QUrl, QAbstractListModel, Qt, \
-        QVariant, QInputDialog
+        QVariant, QFont
 
-from calibre.web.feeds.recipes import compile_recipe
+from calibre.web.feeds.recipes import compile_recipe, custom_recipes
 from calibre.web.feeds.news import AutomaticNewsRecipe
 from calibre.gui2.dialogs.user_profiles_ui import Ui_Dialog
 from calibre.gui2 import error_dialog, question_dialog, open_url, \
-                         choose_files, ResizableDialog, NONE
+                         choose_files, ResizableDialog, NONE, open_local_file
 from calibre.gui2.widgets import PythonHighlighter
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.icu import sort_key
@@ -83,6 +83,9 @@ class UserProfiles(ResizableDialog, Ui_Dialog):
         self._model = self.model = CustomRecipeModel(recipe_model)
         self.available_profiles.setModel(self._model)
         self.available_profiles.currentChanged = self.current_changed
+        f = QFont()
+        f.setStyleHint(f.Monospace)
+        self.source_code.setFont(f)
 
         self.connect(self.remove_feed_button, SIGNAL('clicked(bool)'),
                      self.added_feeds.remove_selected_items)
@@ -93,6 +96,7 @@ class UserProfiles(ResizableDialog, Ui_Dialog):
         self.connect(self.load_button, SIGNAL('clicked()'), self.load)
         self.connect(self.builtin_recipe_button, SIGNAL('clicked()'), self.add_builtin_recipe)
         self.connect(self.share_button, SIGNAL('clicked()'), self.share)
+        self.show_recipe_files_button.clicked.connect(self.show_recipe_files)
         self.connect(self.down_button, SIGNAL('clicked()'), self.down)
         self.connect(self.up_button, SIGNAL('clicked()'), self.up)
         self.connect(self.add_profile_button, SIGNAL('clicked(bool)'),
@@ -102,6 +106,10 @@ class UserProfiles(ResizableDialog, Ui_Dialog):
         self.connect(self.toggle_mode_button, SIGNAL('clicked(bool)'), self.toggle_mode)
         self.clear()
 
+    def show_recipe_files(self, *args):
+        bdir = os.path.dirname(custom_recipes.file_path)
+        open_local_file(bdir)
+
     def break_cycles(self):
         self.recipe_model = self._model.recipe_model = None
         self.available_profiles = None
@@ -256,24 +264,61 @@ class %(classname)s(%(base_class)s):
 
     def add_builtin_recipe(self):
         from calibre.web.feeds.recipes.collection import \
-            get_builtin_recipe_by_title, get_builtin_recipe_titles
-        items = sorted(get_builtin_recipe_titles(), key=sort_key)
+            get_builtin_recipe_collection, get_builtin_recipe_by_id
+        from PyQt4.Qt import QDialog, QVBoxLayout, QListWidgetItem, \
+                QListWidget, QDialogButtonBox, QSize
 
+        d = QDialog(self)
+        d.l = QVBoxLayout()
+        d.setLayout(d.l)
+        d.list = QListWidget(d)
+        d.list.doubleClicked.connect(lambda x: d.accept())
+        d.l.addWidget(d.list)
+        d.bb = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel,
+                Qt.Horizontal, d)
+        d.bb.accepted.connect(d.accept)
+        d.bb.rejected.connect(d.reject)
+        d.l.addWidget(d.bb)
+        d.setWindowTitle(_('Choose builtin recipe'))
+        items = []
+        for r in get_builtin_recipe_collection():
+            id_ = r.get('id', '')
+            title = r.get('title', '')
+            lang = r.get('language', '')
+            if id_ and title:
+                items.append((title + ' [%s]'%lang, id_))
 
-        title, ok = QInputDialog.getItem(self, _('Pick recipe'), _('Pick the recipe to customize'),
-                                     items, 0, False)
-        if ok:
-            title = unicode(title)
-            profile = get_builtin_recipe_by_title(title)
-            if self._model.has_title(title):
-                if question_dialog(self, _('Replace recipe?'),
-                    _('A custom recipe named %s already exists. Do you want to '
-                        'replace it?')%title):
-                    self._model.replace_by_title(title, profile)
-                else:
-                    return
+        items.sort(key=lambda x:sort_key(x[0]))
+        for title, id_ in items:
+            item = QListWidgetItem(title)
+            item.setData(Qt.UserRole, id_)
+            d.list.addItem(item)
+
+        d.resize(QSize(450, 400))
+        ret = d.exec_()
+        d.list.doubleClicked.disconnect()
+        if ret != d.Accepted:
+            return
+
+        items = list(d.list.selectedItems())
+        if not items:
+            return
+        item = items[-1]
+        id_ = unicode(item.data(Qt.UserRole).toString())
+        title = unicode(item.data(Qt.DisplayRole).toString()).rpartition(' [')[0]
+        profile = get_builtin_recipe_by_id(id_)
+        if profile is None:
+            raise Exception('Something weird happened')
+
+        if self._model.has_title(title):
+            if question_dialog(self, _('Replace recipe?'),
+                _('A custom recipe named %s already exists. Do you want to '
+                    'replace it?')%title):
+                self._model.replace_by_title(title, profile)
             else:
-                self.model.add(title, profile)
+                return
+        else:
+            self.model.add(title, profile)
 
         self.clear()
 
@@ -319,11 +364,17 @@ class %(classname)s(%(base_class)s):
         self.populate_options(AutomaticNewsRecipe)
         self.source_code.setText('')
 
+    def reject(self):
+        if question_dialog(self, _('Are you sure?'),
+                _('You will lose any unsaved changes. To save your'
+                    ' changes, click the Add/Update recipe button.'
+                    ' Continue?'), show_copy_button=False):
+            ResizableDialog.reject(self)
+
 if __name__ == '__main__':
     from calibre.gui2 import is_ok_to_use_qt
     is_ok_to_use_qt()
-    from calibre.library.database2 import LibraryDatabase2
     from calibre.web.feeds.recipes.model import RecipeModel
-    d=UserProfiles(None, RecipeModel(LibraryDatabase2('/home/kovid/documents/library')))
+    d=UserProfiles(None, RecipeModel())
     d.exec_()
 
