@@ -22,7 +22,7 @@ from calibre.utils.icu import sort_key, strcmp as icu_strcmp
 from calibre.ebooks.metadata.meta import set_metadata as _set_metadata
 from calibre.utils.search_query_parser import SearchQueryParser
 from calibre.library.caches import _match, CONTAINS_MATCH, EQUALS_MATCH, \
-    REGEXP_MATCH, MetadataBackup
+    REGEXP_MATCH, MetadataBackup, force_to_bool
 from calibre import strftime, isbytestring, prepare_string_for_xml
 from calibre.constants import filesystem_encoding, DEBUG
 from calibre.gui2.library import DEFAULT_SORT
@@ -120,11 +120,10 @@ class BooksModel(QAbstractTableModel): # {{{
 
     def set_device_connected(self, is_connected):
         self.device_connected = is_connected
-        self.refresh_ondevice()
 
     def refresh_ondevice(self):
         self.db.refresh_ondevice()
-        self.refresh() # does a resort()
+        self.resort()
         self.research()
 
     def set_book_on_device_func(self, func):
@@ -238,8 +237,6 @@ class BooksModel(QAbstractTableModel): # {{{
 
     def set_highlight_only(self, toWhat):
         self.highlight_only = toWhat
-        if self.last_search:
-            self.research()
 
     def get_current_highlighted_id(self):
         if len(self.ids_to_highlight) == 0 or self.current_highlighted_idx is None:
@@ -271,6 +268,15 @@ class BooksModel(QAbstractTableModel): # {{{
             return None
         return self.get_current_highlighted_id()
 
+    def highlight_ids(self, ids_to_highlight):
+        self.ids_to_highlight = ids_to_highlight
+        self.ids_to_highlight_set = set(self.ids_to_highlight)
+        if self.ids_to_highlight:
+            self.current_highlighted_idx = 0
+        else:
+            self.current_highlighted_idx = None
+        self.reset()
+
     def search(self, text, reset=True):
         try:
             if self.highlight_only:
@@ -618,7 +624,7 @@ class BooksModel(QAbstractTableModel): # {{{
             return None # displayed using a decorator
 
         def bool_type_decorator(r, idx=-1, bool_cols_are_tristate=True):
-            val = self.db.data[r][idx]
+            val = force_to_bool(self.db.data[r][idx])
             if not bool_cols_are_tristate:
                 if val is None or not val:
                     return self.bool_no_icon
@@ -640,6 +646,14 @@ class BooksModel(QAbstractTableModel): # {{{
                 return QVariant(', '.join(sorted(text.split('|'),key=sort_key)))
             return QVariant(text)
 
+        def decorated_text_type(r, mult=False, idx=-1):
+            text = self.db.data[r][idx]
+            if force_to_bool(text) is not None:
+                return None
+            if text and mult:
+                return QVariant(', '.join(sorted(text.split('|'),key=sort_key)))
+            return QVariant(text)
+
         def number_type(r, idx=-1):
             return QVariant(self.db.data[r][idx])
 
@@ -677,8 +691,16 @@ class BooksModel(QAbstractTableModel): # {{{
             idx = self.custom_columns[col]['rec_index']
             datatype = self.custom_columns[col]['datatype']
             if datatype in ('text', 'comments', 'composite', 'enumeration'):
-                self.dc[col] = functools.partial(text_type, idx=idx,
-                                                 mult=self.custom_columns[col]['is_multiple'])
+                mult=self.custom_columns[col]['is_multiple']
+                self.dc[col] = functools.partial(text_type, idx=idx, mult=mult)
+                if datatype in ['text', 'composite', 'enumeration'] and not mult:
+                    if self.custom_columns[col]['display'].get('use_decorations', False):
+                        self.dc[col] = functools.partial(decorated_text_type,
+                                                         idx=idx, mult=mult)
+                        self.dc_decorator[col] = functools.partial(
+                            bool_type_decorator, idx=idx,
+                            bool_cols_are_tristate=
+                                tweaks['bool_custom_columns_are_tristate'] != 'no')
             elif datatype in ('int', 'float'):
                 self.dc[col] = functools.partial(number_type, idx=idx)
             elif datatype == 'datetime':
@@ -687,7 +709,8 @@ class BooksModel(QAbstractTableModel): # {{{
                 self.dc[col] = functools.partial(bool_type, idx=idx)
                 self.dc_decorator[col] = functools.partial(
                             bool_type_decorator, idx=idx,
-                            bool_cols_are_tristate=tweaks['bool_custom_columns_are_tristate'] == 'yes')
+                            bool_cols_are_tristate=
+                                tweaks['bool_custom_columns_are_tristate'] != 'no')
             elif datatype == 'rating':
                 self.dc[col] = functools.partial(rating_type, idx=idx)
             elif datatype == 'series':
@@ -791,6 +814,16 @@ class BooksModel(QAbstractTableModel): # {{{
                 val = qt_to_dt(val, as_utc=False)
         elif typ == 'series':
             val = unicode(value.toString()).strip()
+            if val:
+                pat = re.compile(r'\[([.0-9]+)\]')
+                match = pat.search(val)
+                if match is not None:
+                    s_index = float(match.group(1))
+                    val = pat.sub('', val).strip()
+                elif val:
+                    if tweaks['series_index_auto_increment'] != 'const':
+                        s_index = self.db.get_next_cc_series_num_for(val,
+                                                        label=label, num=None)
         elif typ == 'composite':
             tmpl = unicode(value.toString()).strip()
             disp = cc['display']
@@ -818,7 +851,7 @@ class BooksModel(QAbstractTableModel): # {{{
                     return False
                 val = int(value.toInt()[0]) if column == 'rating' else \
                       value.toDate() if column in ('timestamp', 'pubdate') else \
-                      unicode(value.toString())
+                      unicode(value.toString()).strip()
                 id = self.db.id(row)
                 books_to_refresh = set([id])
                 if column == 'rating':
@@ -860,11 +893,6 @@ class BooksModel(QAbstractTableModel): # {{{
             self.dataChanged.emit(index, index)
         return True
 
-    def set_search_restriction(self, s):
-        self.db.data.set_search_restriction(s)
-        self.search('')
-        return self.rowCount(None)
-
 # }}}
 
 class OnDeviceSearch(SearchQueryParser): # {{{
@@ -1334,8 +1362,5 @@ class DeviceBooksModel(BooksModel): # {{{
         if prefs['manage_device_metadata']=='on_connect':
             self.editable = []
 
-    def set_search_restriction(self, s):
-        pass
-
 # }}}
 
