@@ -11,7 +11,7 @@ from functools import partial
 from PyQt4.Qt import Qt, QVBoxLayout, QHBoxLayout, QWidget, QPushButton, \
         QGridLayout, pyqtSignal, QDialogButtonBox, QScrollArea, QFont, \
         QTabWidget, QIcon, QToolButton, QSplitter, QGroupBox, QSpacerItem, \
-        QSizePolicy, QPalette, QFrame, QSize
+        QSizePolicy, QPalette, QFrame, QSize, QKeySequence
 
 from calibre.ebooks.metadata import authors_to_string, string_to_authors
 from calibre.gui2 import ResizableDialog, error_dialog, gprefs
@@ -45,9 +45,12 @@ class MetadataSingleDialogBase(ResizableDialog):
         self.button_box.rejected.connect(self.reject)
         self.next_button = QPushButton(QIcon(I('forward.png')), _('Next'),
                 self)
+        self.next_button.setShortcut(QKeySequence('Alt+Right'))
         self.next_button.clicked.connect(partial(self.do_one, delta=1))
         self.prev_button = QPushButton(QIcon(I('back.png')), _('Previous'),
                 self)
+        self.prev_button.setShortcut(QKeySequence('Alt+Left'))
+
         self.button_box.addButton(self.prev_button, self.button_box.ActionRole)
         self.button_box.addButton(self.next_button, self.button_box.ActionRole)
         self.prev_button.clicked.connect(partial(self.do_one, delta=-1))
@@ -197,7 +200,7 @@ class MetadataSingleDialogBase(ResizableDialog):
         self.books_to_refresh = set([])
         for widget in self.basic_metadata_widgets:
             widget.initialize(self.db, id_)
-        for widget in self.custom_metadata_widgets:
+        for widget in getattr(self, 'custom_metadata_widgets', []):
             widget.initialize(id_)
         # Commented out as it doesn't play nice with Next, Prev buttons
         #self.fetch_metadata_button.setFocus(Qt.OtherFocusReason)
@@ -355,11 +358,13 @@ class MetadataSingleDialogBase(ResizableDialog):
             next_ = self.db.title(self.row_list[self.current_row+1])
 
         if next_ is not None:
-            tip = _('Save changes and edit the metadata of %s')%next_
+            tip = (_('Save changes and edit the metadata of %s')+
+                    ' [Alt+Right]')%next_
             self.next_button.setToolTip(tip)
         self.next_button.setVisible(next_ is not None)
         if prev is not None:
-            tip = _('Save changes and edit the metadata of %s')%prev
+            tip = (_('Save changes and edit the metadata of %s')+
+                    ' [Alt+Left]')%prev
             self.prev_button.setToolTip(tip)
         self.prev_button.setVisible(prev is not None)
         self(self.db.id(self.row_list[self.current_row]))
