@@ -16,9 +16,10 @@ from calibre.customize.ui import initialized_plugins, is_disabled, enable_plugin
                                  disable_plugin, plugin_customization, add_plugin, \
                                  remove_plugin
 from calibre.gui2 import NONE, error_dialog, info_dialog, choose_files, \
-        question_dialog
+        question_dialog, gprefs
 from calibre.utils.search_query_parser import SearchQueryParser
 from calibre.utils.icu import lower
+from calibre.utils.ordered_dict import OrderedDict
 
 class PluginModel(QAbstractItemModel, SearchQueryParser): # {{{
 
@@ -109,7 +110,7 @@ class PluginModel(QAbstractItemModel, SearchQueryParser): # {{{
         return self.index(ans[0], 0, QModelIndex()) if ans[1] < 0 else \
                 self.index(ans[1], 0, self.index(ans[0], 0, QModelIndex()))
 
-    def index(self, row, column, parent):
+    def index(self, row, column, parent=QModelIndex()):
         if not self.hasIndex(row, column, parent):
             return QModelIndex()
 
@@ -165,8 +166,6 @@ class PluginModel(QAbstractItemModel, SearchQueryParser): # {{{
     def flags(self, index):
         if not index.isValid():
             return 0
-        if index.internalId() == 0:
-            return Qt.ItemIsEnabled
         flags = Qt.ItemIsSelectable | Qt.ItemIsEnabled
         return flags
 
@@ -283,6 +282,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             self._plugin_model.populate()
             self._plugin_model.reset()
             self.changed_signal.emit()
+            self.check_for_add_to_toolbars(plugin)
             info_dialog(self, _('Success'),
                     _('Plugin <b>{0}</b> successfully installed under <b>'
                         ' {1} plugins</b>. You may have to restart calibre '
@@ -329,7 +329,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                     return error_dialog(self, _('Must restart'),
                             _('You must restart calibre before you can'
                                 ' configure the <b>%s</b> plugin')%plugin.name, show=True)
-                if plugin.do_user_config():
+                if plugin.do_user_config(self.gui):
                     self._plugin_model.refresh_plugin(plugin)
             elif op == 'remove':
                 msg = _('Plugin <b>{0}</b> successfully removed').format(plugin.name)
@@ -344,6 +344,37 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                          plugin.name + _(' cannot be removed. It is a '
                          'builtin plugin. Try disabling it instead.')).exec_()
 
+    def check_for_add_to_toolbars(self, plugin):
+        from calibre.gui2.preferences.toolbar import ConfigWidget
+        from calibre.customize import InterfaceActionBase
+
+        if not isinstance(plugin, InterfaceActionBase):
+            return
+
+        all_locations = OrderedDict(ConfigWidget.LOCATIONS)
+        plugin_action = plugin.load_actual_plugin(self.gui)
+        installed_actions = OrderedDict([
+            (key, list(gprefs.get('action-layout-'+key, [])))
+            for key in all_locations])
+
+        # If already installed in a GUI container, do nothing
+        for action_names in installed_actions.itervalues():
+            if plugin_action.name in action_names:
+                return
+
+        allowed_locations = [(key, text) for key, text in
+                all_locations.iteritems() if key
+                not in plugin_action.dont_add_to]
+        if not allowed_locations:
+            return # This plugin doesn't want to live in the GUI
+
+        from calibre.gui2.dialogs.choose_plugin_toolbars import ChoosePluginToolbarsDialog
+        d = ChoosePluginToolbarsDialog(self, plugin_action, allowed_locations)
+        if d.exec_() == d.Accepted:
+            for key, text in d.selected_locations():
+                installed_actions = list(gprefs.get('action-layout-'+key, []))
+                installed_actions.append(plugin_action.name)
+                gprefs['action-layout-'+key] = tuple(installed_actions)
 
 if __name__ == '__main__':
     from PyQt4.Qt import QApplication
