@@ -16,7 +16,6 @@ from calibre.gui2 import config
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.dialogs.saved_search_editor import SavedSearchEditor
 from calibre.gui2.dialogs.search import SearchDialog
-from calibre.utils.config import dynamic
 from calibre.utils.search_query_parser import saved_searches
 from calibre.utils.icu import sort_key
 
@@ -115,6 +114,9 @@ class SearchBox2(QComboBox): # {{{
     def text(self):
         return self.currentText()
 
+    def clear_history(self, *args):
+        QComboBox.clear(self)
+
     def clear(self, emit_search=True):
         self.normalize_state()
         self.setEditText('')
@@ -215,11 +217,15 @@ class SearchBox2(QComboBox): # {{{
                 self.clear()
             else:
                 self.normalize_state()
+                self.lineEdit().setCompleter(None)
                 self.setEditText(txt)
                 self.line_edit.end(False)
                 if emit_changed:
                     self.changed.emit()
                 self._do_search(store_in_history=store_in_history)
+                c = QCompleter()
+                self.lineEdit().setCompleter(c)
+                c.setCompletionMode(c.PopupCompletion)
             self.focus_to_library.emit()
         finally:
             if not store_in_history:
@@ -271,7 +277,7 @@ class SavedSearchBox(QComboBox): # {{{
     def initialize(self, _search_box, colorize=False, help_text=_('Search')):
         self.search_box = _search_box
         try:
-           self.line_edit.setPlaceholderText(help_text)
+            self.line_edit.setPlaceholderText(help_text)
         except:
             # Using Qt < 4.7
             pass
@@ -376,9 +382,7 @@ class SearchBoxMixin(object): # {{{
             unicode(self.search.toolTip())))
         self.advanced_search_button.setStatusTip(self.advanced_search_button.toolTip())
         self.clear_button.setStatusTip(self.clear_button.toolTip())
-        self.search_highlight_only.stateChanged.connect(self.highlight_only_changed)
-        self.search_highlight_only.setChecked(
-                            dynamic.get('search_highlight_only', False))
+        self.search_options_button.clicked.connect(self.search_options_button_clicked)
 
     def focus_search_box(self, *args):
         self.search.setFocus(Qt.OtherFocusReason)
@@ -402,14 +406,13 @@ class SearchBoxMixin(object): # {{{
         self.search.do_search()
         self.focus_to_library()
 
+    def search_options_button_clicked(self):
+        self.iactions['Preferences'].do_config(initial_plugin=('Interface',
+            'Search'), close_after_initial=True)
+
     def focus_to_library(self):
         self.current_view().setFocus(Qt.OtherFocusReason)
 
-    def highlight_only_changed(self, toWhat):
-        dynamic.set('search_highlight_only', toWhat)
-        self.current_view().model().set_highlight_only(toWhat)
-        self.focus_to_library()
-
     # }}}
 
 class SavedSearchBoxMixin(object): # {{{
@@ -433,24 +436,28 @@ class SavedSearchBoxMixin(object): # {{{
             b = getattr(self, x+'_search_button')
             b.setStatusTip(b.toolTip())
 
-    def saved_searches_changed(self):
+    def saved_searches_changed(self, set_restriction=None):
         p = sorted(saved_searches().names(), key=sort_key)
-        t = unicode(self.search_restriction.currentText())
+        if set_restriction is None:
+            set_restriction = unicode(self.search_restriction.currentText())
         # rebuild the restrictions combobox using current saved searches
         self.search_restriction.clear()
         self.search_restriction.addItem('')
         self.tags_view.recount()
         for s in p:
             self.search_restriction.addItem(s)
-        if t: # redo the search restriction if there was one
-            self.apply_named_search_restriction(t)
+        if set_restriction: # redo the search restriction if there was one
+            self.apply_named_search_restriction(set_restriction)
 
     def do_saved_search_edit(self, search):
         d = SavedSearchEditor(self, search)
         d.exec_()
         if d.result() == d.Accepted:
-            self.saved_searches_changed()
-            self.saved_search.clear()
+            self.do_rebuild_saved_searches()
+
+    def do_rebuild_saved_searches(self):
+        self.saved_searches_changed()
+        self.saved_search.clear()
 
     # }}}
 
