@@ -9,7 +9,7 @@ __docformat__ = 'restructuredtext en'
 
 '''The main GUI'''
 
-import collections, os, sys, textwrap, time
+import collections, os, sys, textwrap, time, gc
 from Queue import Queue, Empty
 from threading import Thread
 from PyQt4.Qt import Qt, SIGNAL, QTimer, QHelpEvent, QAction, \
@@ -95,7 +95,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
 
 
     def __init__(self, opts, parent=None, gui_debug=None):
-        MainWindow.__init__(self, opts, parent)
+        MainWindow.__init__(self, opts, parent=parent, disable_automatic_gc=True)
         self.opts = opts
         self.device_connected = None
         self.gui_debug = gui_debug
@@ -166,10 +166,6 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
         EmailMixin.__init__(self)
         DeviceMixin.__init__(self)
 
-        self.restriction_count_of_books_in_view = 0
-        self.restriction_count_of_books_in_library = 0
-        self.restriction_in_effect = False
-
         self.progress_indicator = ProgressIndicator(self)
         self.progress_indicator.pos = (0, 20)
         self.verbose = opts.verbose
@@ -300,6 +296,10 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
                 traceback.print_exc()
                 if ac.plugin_path is None:
                     raise
+        self.device_manager.set_current_library_uuid(db.library_id)
+
+        # Collect cycles now
+        gc.collect()
 
         if show_gui and self.gui_debug is not None:
             info_dialog(self, _('Debug mode'), '<p>' +
@@ -311,7 +311,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
     def esc(self, *args):
         self.search.clear()
 
-    def start_content_server(self):
+    def start_content_server(self, check_started=True):
         from calibre.library.server.main import start_threaded_server
         from calibre.library.server import server_config
         self.content_server = start_threaded_server(
@@ -319,7 +319,8 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
         self.content_server.state_callback = Dispatcher(
                 self.iactions['Connect Share'].content_server_state_changed)
         self.content_server.state_callback(True)
-        self.test_server_timer = QTimer.singleShot(10000, self.test_server)
+        if check_started:
+            QTimer.singleShot(10000, self.test_server)
 
     def resizeEvent(self, ev):
         MainWindow.resizeEvent(self, ev)
@@ -401,6 +402,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
         elif msg.startswith('refreshdb:'):
             self.library_view.model().refresh()
             self.library_view.model().research()
+            self.tags_view.recount()
         else:
             print msg
 
@@ -464,6 +466,10 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
             self.memory_view.reset()
             self.card_a_view.reset()
             self.card_b_view.reset()
+        self.device_manager.set_current_library_uuid(db.library_id)
+        # Run a garbage collection now so that it does not freeze the
+        # interface later
+        gc.collect()
 
 
     def set_window_title(self):
@@ -483,8 +489,10 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
             action.location_selected(location)
         if location == 'library':
             self.search_restriction.setEnabled(True)
+            self.search_options_button.setEnabled(True)
         else:
             self.search_restriction.setEnabled(False)
+            self.search_options_button.setEnabled(False)
             # Reset the view in case something changed while it was invisible
             self.current_view().reset()
         self.set_number_of_books_shown()
@@ -634,7 +642,8 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
             mb.stop()
 
         self.hide_windows()
-        self.emailer.stop()
+        if self.emailer.is_alive():
+            self.emailer.stop()
         try:
             try:
                 if self.content_server is not None:
@@ -647,6 +656,8 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin, # {{{
             pass
         time.sleep(2)
         self.hide_windows()
+        # Do not report any errors that happen after the shutdown
+        sys.excepthook = sys.__excepthook__
         return True
 
     def run_wizard(self, *args):
