@@ -17,16 +17,18 @@ from calibre.gui2.viewer.bookmarkmanager import BookmarkManager
 from calibre.gui2.widgets import ProgressIndicator
 from calibre.gui2.main_window import MainWindow
 from calibre.gui2 import Application, ORG_NAME, APP_UID, choose_files, \
-                         info_dialog, error_dialog, open_url, available_height
+    info_dialog, error_dialog, open_url, available_height
 from calibre.ebooks.oeb.iterator import EbookIterator
 from calibre.ebooks import DRMError
-from calibre.constants import islinux, isfreebsd, isosx
-from calibre.utils.config import Config, StringConfig, dynamic
+from calibre.constants import islinux, isfreebsd, isosx, filesystem_encoding
+from calibre.utils.config import Config, StringConfig, JSONConfig
 from calibre.gui2.search_box import SearchBox2
 from calibre.ebooks.metadata import MetaInformation
 from calibre.customize.ui import available_input_formats
 from calibre.gui2.viewer.dictionary import Lookup
-from calibre import as_unicode
+from calibre import as_unicode, force_unicode, isbytestring
+
+vprefs = JSONConfig('viewer')
 
 class TOCItem(QStandardItem):
 
@@ -160,6 +162,12 @@ class HelpfulLineEdit(QLineEdit):
         self.setPalette(self.gray)
         self.setText(self.HELP_TEXT)
 
+class RecentAction(QAction):
+
+    def __init__(self, path, parent):
+        self.path = path
+        QAction.__init__(self, os.path.basename(path), parent)
+
 class EbookViewer(MainWindow, Ui_EbookViewer):
 
     STATE_VERSION = 1
@@ -217,6 +225,12 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         self.action_quit.setShortcuts(qs)
         self.connect(self.action_quit, SIGNAL('triggered(bool)'),
                      lambda x:QApplication.instance().quit())
+        self.action_focus_search = QAction(self)
+        self.addAction(self.action_focus_search)
+        self.action_focus_search.setShortcuts([Qt.Key_Slash,
+            QKeySequence(QKeySequence.Find)])
+        self.action_focus_search.triggered.connect(lambda x:
+                self.search.setFocus(Qt.OtherFocusReason))
         self.action_copy.setDisabled(True)
         self.action_metadata.setCheckable(True)
         self.action_metadata.setShortcut(Qt.CTRL+Qt.Key_I)
@@ -284,24 +298,53 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         ca = self.view.copy_action
         ca.setShortcut(QKeySequence.Copy)
         self.addAction(ca)
+        self.open_history_menu = QMenu()
+        self.clear_recent_history_action = QAction(
+                _('Clear list of recently opened books'), self)
+        self.clear_recent_history_action.triggered.connect(self.clear_recent_history)
+        self.build_recent_menu()
+        self.action_open_ebook.setMenu(self.open_history_menu)
+        self.open_history_menu.triggered[QAction].connect(self.open_recent)
+        w = self.tool_bar.widgetForAction(self.action_open_ebook)
+        w.setPopupMode(QToolButton.MenuButtonPopup)
+
         self.restore_state()
 
+    def clear_recent_history(self, *args):
+        vprefs.set('viewer_open_history', [])
+        self.build_recent_menu()
+
+    def build_recent_menu(self):
+        m = self.open_history_menu
+        m.clear()
+        recent = vprefs.get('viewer_open_history', [])
+        if recent:
+            m.addAction(self.clear_recent_history_action)
+            m.addSeparator()
+        count = 0
+        for path in recent:
+            if count > 9:
+                break
+            if os.path.exists(path):
+                m.addAction(RecentAction(path, m))
+                count += 1
+
     def closeEvent(self, e):
         self.save_state()
         return MainWindow.closeEvent(self, e)
 
     def save_state(self):
-        state = str(self.saveState(self.STATE_VERSION))
-        dynamic['viewer_toolbar_state'] = state
-        dynamic.set('viewer_window_geometry', self.saveGeometry())
+        state = bytearray(self.saveState(self.STATE_VERSION))
+        vprefs['viewer_toolbar_state'] = state
+        vprefs.set('viewer_window_geometry', bytearray(self.saveGeometry()))
         if self.current_book_has_toc:
-            dynamic.set('viewer_toc_isvisible', bool(self.toc.isVisible()))
+            vprefs.set('viewer_toc_isvisible', bool(self.toc.isVisible()))
         if self.toc.isVisible():
-            dynamic.set('viewer_splitter_state',
+            vprefs.set('viewer_splitter_state',
                 bytearray(self.splitter.saveState()))
 
     def restore_state(self):
-        state = dynamic.get('viewer_toolbar_state', None)
+        state = vprefs.get('viewer_toolbar_state', None)
         if state is not None:
             try:
                 state = QByteArray(state)
@@ -425,6 +468,9 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         if files:
             self.load_ebook(files[0])
 
+    def open_recent(self, action):
+        self.load_ebook(action.path)
+
     def font_size_larger(self, checked):
         frac = self.view.magnify_fonts()
         self.action_font_size_larger.setEnabled(self.view.multiplier() < 3)
@@ -465,12 +511,6 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         if self.view.search(text, backwards=backwards):
             self.scrolled(self.view.scroll_fraction)
 
-    def keyPressEvent(self, event):
-        if event.key() == Qt.Key_Slash:
-            self.search.setFocus(Qt.OtherFocusReason)
-        else:
-            return MainWindow.keyPressEvent(self, event)
-
     def internal_link_clicked(self, frac):
         self.history.add(self.pos.value())
 
@@ -647,6 +687,17 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
                     self.action_table_of_contents.setChecked(True)
             else:
                 self.action_table_of_contents.setChecked(False)
+            if isbytestring(pathtoebook):
+                pathtoebook = force_unicode(pathtoebook, filesystem_encoding)
+            vh = vprefs.get('viewer_open_history', [])
+            try:
+                vh.remove(pathtoebook)
+            except:
+                pass
+            vh.insert(0, pathtoebook)
+            vprefs.set('viewer_open_history', vh[:50])
+            self.build_recent_menu()
+
             self.action_table_of_contents.setDisabled(not self.iterator.toc)
             self.current_book_has_toc = bool(self.iterator.toc)
             self.current_title = title
@@ -701,13 +752,13 @@ class EbookViewer(MainWindow, Ui_EbookViewer):
         c = config().parse()
         self.splitter.setSizes([1, 300])
         if c.remember_window_size:
-            wg = dynamic.get('viewer_window_geometry', None)
+            wg = vprefs.get('viewer_window_geometry', None)
             if wg is not None:
                 self.restoreGeometry(wg)
-            ss = dynamic.get('viewer_splitter_state', None)
+            ss = vprefs.get('viewer_splitter_state', None)
             if ss is not None:
                 self.splitter.restoreState(ss)
-            self.show_toc_on_open = dynamic.get('viewer_toc_isvisible', False)
+            self.show_toc_on_open = vprefs.get('viewer_toc_isvisible', False)
         av = available_height() - 30
         if self.height() > av:
             self.resize(self.width(), av)
