@@ -3,7 +3,7 @@ Created on 25 May 2010
 
 @author: charles
 '''
-import copy
+import copy, traceback
 
 from calibre.utils.ordered_dict import OrderedDict
 from calibre.utils.config import tweaks
@@ -16,7 +16,8 @@ class TagsIcons(dict):
     '''
 
     category_icons = ['authors', 'series', 'formats', 'publisher', 'rating',
-                      'news',    'tags',   ':custom', ':user',     'search',]
+                      'news',    'tags',   'custom:', 'user:',     'search',
+                      'identifiers']
     def __init__(self, icon_dict):
         for a in self.category_icons:
             if a not in icon_dict:
@@ -24,16 +25,17 @@ class TagsIcons(dict):
             self[a] = icon_dict[a]
 
 category_icon_map = {
-                    'authors'   : 'user_profile.png',
-                    'series'    : 'series.png',
-                    'formats'   : 'book.png',
-                    'publisher' : 'publisher.png',
-                    'rating'    : 'rating.png',
-                    'news'      : 'news.png',
-                    'tags'      : 'tags.png',
-                    ':custom'   : 'column.png',
-                    ':user'     : 'drawer.png',
-                    'search'    : 'search.png'
+                    'authors'    : 'user_profile.png',
+                    'series'     : 'series.png',
+                    'formats'    : 'book.png',
+                    'publisher'  : 'publisher.png',
+                    'rating'     : 'rating.png',
+                    'news'       : 'news.png',
+                    'tags'       : 'tags.png',
+                    'custom:'    : 'column.png',
+                    'user:'      : 'tb_folder.png',
+                    'search'     : 'search.png',
+                    'identifiers': 'id_card.png'
             }
 
 
@@ -80,6 +82,8 @@ class FieldMetadata(dict):
 
     rec_index: the index of the field in the db metadata record.
 
+    is_csp: field contains colon-separated pairs. Must also be text, is_multiple
+
     '''
 
     VALID_DATA_TYPES = frozenset([None, 'rating', 'text', 'comments', 'datetime',
@@ -98,7 +102,8 @@ class FieldMetadata(dict):
                            'name':_('Authors'),
                            'search_terms':['authors', 'author'],
                            'is_custom':False,
-                           'is_category':True}),
+                           'is_category':True,
+                           'is_csp': False}),
             ('series',    {'table':'series',
                            'column':'name',
                            'link_column':'series',
@@ -109,7 +114,8 @@ class FieldMetadata(dict):
                            'name':_('Series'),
                            'search_terms':['series'],
                            'is_custom':False,
-                           'is_category':True}),
+                           'is_category':True,
+                           'is_csp': False}),
             ('formats',   {'table':None,
                            'column':None,
                            'datatype':'text',
@@ -118,7 +124,8 @@ class FieldMetadata(dict):
                            'name':_('Formats'),
                            'search_terms':['formats', 'format'],
                            'is_custom':False,
-                           'is_category':True}),
+                           'is_category':True,
+                           'is_csp': False}),
             ('publisher', {'table':'publishers',
                            'column':'name',
                            'link_column':'publisher',
@@ -129,7 +136,8 @@ class FieldMetadata(dict):
                            'name':_('Publishers'),
                            'search_terms':['publisher'],
                            'is_custom':False,
-                           'is_category':True}),
+                           'is_category':True,
+                           'is_csp': False}),
             ('rating',    {'table':'ratings',
                            'column':'rating',
                            'link_column':'rating',
@@ -140,7 +148,8 @@ class FieldMetadata(dict):
                            'name':_('Ratings'),
                            'search_terms':['rating'],
                            'is_custom':False,
-                           'is_category':True}),
+                           'is_category':True,
+                           'is_csp': False}),
             ('news',      {'table':'news',
                            'column':'name',
                            'category_sort':'name',
@@ -150,7 +159,8 @@ class FieldMetadata(dict):
                            'name':_('News'),
                            'search_terms':[],
                            'is_custom':False,
-                           'is_category':True}),
+                           'is_category':True,
+                           'is_csp': False}),
             ('tags',      {'table':'tags',
                            'column':'name',
                            'link_column': 'tag',
@@ -161,7 +171,18 @@ class FieldMetadata(dict):
                            'name':_('Tags'),
                            'search_terms':['tags', 'tag'],
                            'is_custom':False,
-                           'is_category':True}),
+                           'is_category':True,
+                           'is_csp': False}),
+            ('identifiers',   {'table':None,
+                           'column':None,
+                           'datatype':'text',
+                           'is_multiple':',',
+                           'kind':'field',
+                           'name':_('Identifiers'),
+                           'search_terms':['identifiers', 'identifier', 'isbn'],
+                           'is_custom':False,
+                           'is_category':True,
+                           'is_csp': True}),
             ('author_sort',{'table':None,
                             'column':None,
                             'datatype':'text',
@@ -170,7 +191,8 @@ class FieldMetadata(dict):
                            'name':None,
                            'search_terms':['author_sort'],
                            'is_custom':False,
-                           'is_category':False}),
+                           'is_category':False,
+                           'is_csp': False}),
             ('au_map',    {'table':None,
                            'column':None,
                            'datatype':'text',
@@ -179,7 +201,8 @@ class FieldMetadata(dict):
                            'name':None,
                            'search_terms':[],
                            'is_custom':False,
-                           'is_category':False}),
+                           'is_category':False,
+                           'is_csp': False}),
             ('comments',  {'table':None,
                            'column':None,
                            'datatype':'text',
@@ -187,7 +210,9 @@ class FieldMetadata(dict):
                            'kind':'field',
                            'name':_('Comments'),
                            'search_terms':['comments', 'comment'],
-                           'is_custom':False, 'is_category':False}),
+                           'is_custom':False,
+                           'is_category':False,
+                           'is_csp': False}),
             ('cover',     {'table':None,
                            'column':None,
                            'datatype':'int',
@@ -196,16 +221,8 @@ class FieldMetadata(dict):
                            'name':None,
                            'search_terms':['cover'],
                            'is_custom':False,
-                           'is_category':False}),
-            ('flags',     {'table':None,
-                           'column':None,
-                           'datatype':'text',
-                           'is_multiple':None,
-                           'kind':'field',
-                           'name':None,
-                           'search_terms':[],
-                           'is_custom':False,
-                           'is_category':False}),
+                           'is_category':False,
+                           'is_csp': False}),
             ('id',        {'table':None,
                            'column':None,
                            'datatype':'int',
@@ -214,25 +231,18 @@ class FieldMetadata(dict):
                            'name':None,
                            'search_terms':[],
                            'is_custom':False,
-                           'is_category':False}),
-            ('isbn',      {'table':None,
+                           'is_category':False,
+                           'is_csp': False}),
+            ('last_modified', {'table':None,
                            'column':None,
-                           'datatype':'text',
+                           'datatype':'datetime',
                            'is_multiple':None,
                            'kind':'field',
-                           'name':None,
-                           'search_terms':['isbn'],
+                           'name':_('Date'),
+                           'search_terms':['last_modified'],
                            'is_custom':False,
-                           'is_category':False}),
-            ('lccn',      {'table':None,
-                           'column':None,
-                           'datatype':'text',
-                           'is_multiple':None,
-                           'kind':'field',
-                           'name':None,
-                           'search_terms':[],
-                           'is_custom':False,
-                           'is_category':False}),
+                           'is_category':False,
+                           'is_csp': False}),
             ('ondevice',  {'table':None,
                            'column':None,
                            'datatype':'text',
@@ -241,7 +251,8 @@ class FieldMetadata(dict):
                            'name':_('On Device'),
                            'search_terms':['ondevice'],
                            'is_custom':False,
-                           'is_category':False}),
+                           'is_category':False,
+                           'is_csp': False}),
             ('path',      {'table':None,
                            'column':None,
                            'datatype':'text',
@@ -250,7 +261,8 @@ class FieldMetadata(dict):
                            'name':None,
                            'search_terms':[],
                            'is_custom':False,
-                           'is_category':False}),
+                           'is_category':False,
+                           'is_csp': False}),
             ('pubdate',   {'table':None,
                            'column':None,
                            'datatype':'datetime',
@@ -259,7 +271,18 @@ class FieldMetadata(dict):
                            'name':_('Published'),
                            'search_terms':['pubdate'],
                            'is_custom':False,
-                           'is_category':False}),
+                           'is_category':False,
+                           'is_csp': False}),
+            ('marked',    {'table':None,
+                           'column':None,
+                           'datatype':'text',
+                           'is_multiple':None,
+                           'kind':'field',
+                           'name': None,
+                           'search_terms':['marked'],
+                           'is_custom':False,
+                           'is_category':False,
+                           'is_csp': False}),
             ('series_index',{'table':None,
                              'column':None,
                              'datatype':'float',
@@ -268,7 +291,8 @@ class FieldMetadata(dict):
                              'name':None,
                              'search_terms':['series_index'],
                              'is_custom':False,
-                             'is_category':False}),
+                             'is_category':False,
+                           'is_csp': False}),
             ('sort',      {'table':None,
                            'column':None,
                            'datatype':'text',
@@ -277,7 +301,8 @@ class FieldMetadata(dict):
                            'name':_('Title Sort'),
                            'search_terms':['title_sort'],
                            'is_custom':False,
-                           'is_category':False}),
+                           'is_category':False,
+                           'is_csp': False}),
             ('size',      {'table':None,
                            'column':None,
                            'datatype':'float',
@@ -286,7 +311,8 @@ class FieldMetadata(dict):
                            'name':_('Size (MB)'),
                            'search_terms':['size'],
                            'is_custom':False,
-                           'is_category':False}),
+                           'is_category':False,
+                           'is_csp': False}),
             ('timestamp', {'table':None,
                            'column':None,
                            'datatype':'datetime',
@@ -295,7 +321,8 @@ class FieldMetadata(dict):
                            'name':_('Date'),
                            'search_terms':['date'],
                            'is_custom':False,
-                           'is_category':False}),
+                           'is_category':False,
+                           'is_csp': False}),
             ('title',     {'table':None,
                            'column':None,
                            'datatype':'text',
@@ -304,7 +331,8 @@ class FieldMetadata(dict):
                            'name':_('Title'),
                            'search_terms':['title'],
                            'is_custom':False,
-                           'is_category':False}),
+                           'is_category':False,
+                           'is_csp': False}),
             ('uuid',      {'table':None,
                            'column':None,
                            'datatype':'text',
@@ -313,7 +341,8 @@ class FieldMetadata(dict):
                            'name':None,
                            'search_terms':[],
                            'is_custom':False,
-                           'is_category':False}),
+                           'is_category':False,
+                           'is_csp': False}),
         ]
     # }}}
 
@@ -335,7 +364,8 @@ class FieldMetadata(dict):
             self._tb_cats[k]['display'] = {}
             self._tb_cats[k]['is_editable'] = True
             self._add_search_terms_to_map(k, v['search_terms'])
-        self._tb_cats['timestamp']['display'] = {
+        for x in ('timestamp', 'last_modified'):
+            self._tb_cats[x]['display'] = {
                         'date_format': tweaks['gui_timestamp_display_format']}
         self._tb_cats['pubdate']['display'] = {
                         'date_format': tweaks['gui_pubdate_display_format']}
@@ -441,7 +471,8 @@ class FieldMetadata(dict):
         return l
 
     def add_custom_field(self, label, table, column, datatype, colnum, name,
-                               display, is_editable, is_multiple, is_category):
+                         display, is_editable, is_multiple, is_category,
+                         is_csp=False):
         key = self.custom_field_prefix + label
         if key in self._tb_cats:
             raise ValueError('Duplicate custom field [%s]'%(label))
@@ -454,7 +485,7 @@ class FieldMetadata(dict):
                              'colnum':colnum,      'display':display,
                              'is_custom':True,     'is_category':is_category,
                              'link_column':'value','category_sort':'value',
-                             'is_editable': is_editable,}
+                             'is_csp' : is_csp,     'is_editable': is_editable,}
         self._add_search_terms_to_map(key, [key])
         self.custom_label_to_key_map[label] = key
         if datatype == 'series':
@@ -466,7 +497,7 @@ class FieldMetadata(dict):
                                  'colnum':None,        'display':{},
                                  'is_custom':False,    'is_category':False,
                                  'link_column':None,   'category_sort':None,
-                                 'is_editable': False,}
+                                 'is_editable': False, 'is_csp': False}
             self._add_search_terms_to_map(key, [key])
             self.custom_label_to_key_map[label+'_index'] = key
 
@@ -474,19 +505,49 @@ class FieldMetadata(dict):
         for key in list(self._tb_cats.keys()):
             val = self._tb_cats[key]
             if val['is_category'] and val['kind'] in ('user', 'search'):
+                for k in self._tb_cats[key]['search_terms']:
+                    if k in self._search_term_map:
+                        del self._search_term_map[k]
                 del self._tb_cats[key]
 
+    def remove_user_categories(self):
+        for key in list(self._tb_cats.keys()):
+            val = self._tb_cats[key]
+            if val['is_category'] and val['kind']  == 'user':
+                for k in self._tb_cats[key]['search_terms']:
+                    if k in self._search_term_map:
+                        del self._search_term_map[k]
+                del self._tb_cats[key]
+
+    def _remove_grouped_search_terms(self):
+        to_remove = [v for v in self._search_term_map
+                        if isinstance(self._search_term_map[v], list)]
+        for v in to_remove:
+            del self._search_term_map[v]
+
+    def add_grouped_search_terms(self, gst):
+        self._remove_grouped_search_terms()
+        for t in gst:
+            try:
+                self._add_search_terms_to_map(gst[t], [t])
+            except ValueError:
+                traceback.print_exc()
+
     def cc_series_index_column_for(self, key):
         return self._tb_cats[key]['rec_index'] + 1
 
     def add_user_category(self, label, name):
         if label in self._tb_cats:
             raise ValueError('Duplicate user field [%s]'%(label))
-        self._tb_cats[label] = {'table':None,        'column':None,
-                                'datatype':None,     'is_multiple':None,
-                                'kind':'user',       'name':name,
-                                'search_terms':[],    'is_custom':False,
-                                'is_category':True}
+        st = [label]
+        if icu_lower(label) != label:
+            st.append(icu_lower(label))
+        self._tb_cats[label] = {'table':None,          'column':None,
+                                'datatype':None,       'is_multiple':None,
+                                'kind':'user',         'name':name,
+                                'search_terms':st,     'is_custom':False,
+                                'is_category':True,    'is_csp': False}
+        self._add_search_terms_to_map(label, st)
 
     def add_search_category(self, label, name):
         if label in self._tb_cats:
@@ -494,8 +555,8 @@ class FieldMetadata(dict):
         self._tb_cats[label] = {'table':None,        'column':None,
                                 'datatype':None,     'is_multiple':None,
                                 'kind':'search',     'name':name,
-                                'search_terms':[],    'is_custom':False,
-                                'is_category':True}
+                                'search_terms':[],   'is_custom':False,
+                                'is_category':True,  'is_csp': False}
 
     def set_field_record_index(self, label, index, prefer_custom=False):
         if prefer_custom:
@@ -518,7 +579,6 @@ class FieldMetadata(dict):
     def _add_search_terms_to_map(self, key, terms):
         if terms is not None:
             for t in terms:
-                t = t.lower()
                 if t in self._search_term_map:
                     raise ValueError('Attempt to add duplicate search term "%s"'%t)
                 self._search_term_map[t] = key
