@@ -12,7 +12,7 @@ import cherrypy
 
 from calibre.constants import filesystem_encoding
 from calibre import isbytestring, force_unicode, fit_image, \
-        prepare_string_for_xml as xml
+        prepare_string_for_xml
 from calibre.utils.ordered_dict import OrderedDict
 from calibre.utils.filenames import ascii_filename
 from calibre.utils.config import prefs, tweaks
@@ -23,6 +23,10 @@ from calibre.library.server import custom_fields_to_display
 from calibre.library.field_metadata import category_icon_map
 from calibre.library.server.utils import quote, unquote
 
+def xml(*args, **kwargs):
+    ans = prepare_string_for_xml(*args, **kwargs)
+    return ans.replace('&apos;', '&#39;')
+
 def render_book_list(ids, prefix, suffix=''): # {{{
     pages = []
     num = len(ids)
@@ -342,10 +346,11 @@ class BrowseServer(object):
             return category_meta[x]['name'].lower()
 
         displayed_custom_fields = custom_fields_to_display(self.db)
+        uc_displayed = set()
         for category in sorted(categories, key=lambda x: sort_key(getter(x))):
             if len(categories[category]) == 0:
                 continue
-            if category == 'formats':
+            if category in ('formats', 'identifiers'):
                 continue
             meta = category_meta.get(category, None)
             if meta is None:
@@ -361,7 +366,19 @@ class BrowseServer(object):
                 icon = category_icon_map['user:']
             else:
                 icon = 'blank.png'
-            cats.append((meta['name'], category, icon))
+
+            if meta['kind'] == 'user':
+                dot = category.find('.')
+                if dot > 0:
+                    cat = category[:dot]
+                    if cat not in uc_displayed:
+                        cats.append((meta['name'][:dot-1], cat, icon))
+                        uc_displayed.add(cat)
+                else:
+                    cats.append((meta['name'], category, icon))
+                    uc_displayed.add(category)
+            else:
+                cats.append((meta['name'], category, icon))
 
         cats = [(u'<li><a title="{2} {0}" href="{3}/browse/category/{1}">&nbsp;</a>'
                  u'<img src="{3}{src}" alt="{0}" />'
@@ -394,13 +411,59 @@ class BrowseServer(object):
         category_name = category_meta[category]['name']
         datatype = category_meta[category]['datatype']
 
+        # See if we have any sub-categories to display. As we find them, add
+        # them to the displayed set to avoid showing the same item twice
+        uc_displayed = set()
+        cats = []
+        for ucat in sorted(categories.keys(), key=sort_key):
+            if len(categories[ucat]) == 0:
+                continue
+            if category == 'formats':
+                continue
+            meta = category_meta.get(ucat, None)
+            if meta is None:
+                continue
+            if meta['kind'] != 'user':
+                continue
+            cat_len = len(category)
+            if not (len(ucat) > cat_len and ucat.startswith(category+'.')):
+                continue
+            icon = category_icon_map['user:']
+            # we have a subcategory. Find any further dots (further subcats)
+            cat_len += 1
+            cat = ucat[cat_len:]
+            dot = cat.find('.')
+            if dot > 0:
+                # More subcats
+                cat = cat[:dot]
+                if cat not in uc_displayed:
+                    cats.append((cat, ucat[:cat_len+dot], icon))
+                    uc_displayed.add(cat)
+            else:
+                # This is the end of the chain
+                cats.append((cat, ucat, icon))
+                uc_displayed.add(cat)
 
+        cats = u'\n\n'.join(
+                [(u'<li><a title="{2} {0}" href="{3}/browse/category/{1}">&nbsp;</a>'
+                 u'<img src="{3}{src}" alt="{0}" />'
+                 u'<span class="label">{0}</span>'
+                 u'</li>')
+                .format(xml(x, True), xml(quote(y)), xml(_('Browse books by')),
+                    self.opts.url_prefix, src='/browse/icon/'+z)
+                for x, y, z in cats])
+        if cats:
+            cats = (u'\n<div class="toplevel">\n'
+                     '{0}</div>').format(cats)
+            script = 'toplevel();'
+        else:
+            script = 'true'
+
+        # Now do the category items
         items = categories[category]
         sort = self.browse_sort_categories(items, sort)
 
-        script = 'true'
-
-        if len(items) == 1:
+        if not cats and len(items) == 1:
             # Only one item in category, go directly to book list
             prefix = '' if self.is_wsgi else self.opts.url_prefix
             html = get_category_items(category, items,
@@ -443,7 +506,10 @@ class BrowseServer(object):
 
 
 
-        script = 'category(%s);'%script
+        if cats:
+            script = 'toplevel();category(%s);'%script
+        else:
+            script = 'category(%s);'%script
 
         main = u'''
             <div class="category">
@@ -453,7 +519,7 @@ class BrowseServer(object):
                 {1}
             </div>
         '''.format(
-                xml(_('Browsing by')+': ' + category_name), items,
+                xml(_('Browsing by')+': ' + category_name), cats + items,
                 xml(_('Up'), True), self.opts.url_prefix)
 
         return self.browse_template(sort).format(title=category_name,
@@ -564,6 +630,8 @@ class BrowseServer(object):
             elif category == 'allbooks':
                 ids = all_ids
             else:
+                if fm.get(category, {'datatype':None})['datatype'] == 'composite':
+                    cid = cid.decode('utf-8')
                 q = category
                 if q == 'news':
                     q = 'tags'
@@ -584,7 +652,7 @@ class BrowseServer(object):
                 title=_('Books in') + " " +category_name,
                 script='booklist(%s);'%hide_sort, main=html)
 
-    def browse_get_book_args(self, mi, id_):
+    def browse_get_book_args(self, mi, id_, add_category_links=False):
         fmts = self.db.formats(id_, index_is_id=True)
         if not fmts:
             fmts = ''
@@ -596,11 +664,45 @@ class BrowseServer(object):
             fmt = None
         args = {'id':id_, 'mi':mi,
                 }
+        ccache = self.categories_cache() if add_category_links else {}
         for key in mi.all_field_keys():
             val = mi.format_field(key)[1]
             if not val:
                 val = ''
-            args[key] = xml(val, True)
+            if add_category_links:
+                added_key = False
+                fm = mi.metadata_for_field(key)
+                if val and fm and fm['is_category'] and not fm['is_csp'] and\
+                        key != 'formats' and fm['datatype'] not in ['rating']:
+                    categories = mi.get(key)
+                    if isinstance(categories, basestring):
+                        categories = [categories]
+                    dbtags = []
+                    for category in categories:
+                        dbtag = None
+                        for tag in ccache[key]:
+                            if tag.name == category:
+                                dbtag = tag
+                                break
+                        dbtags.append(dbtag)
+                    if None not in dbtags:
+                        vals = []
+                        for tag in dbtags:
+                            tval = ('<a title="Browse books by {3}: {0}"'
+                            ' href="{1}" class="details_category_link">{2}</a>')
+                            href='/browse/matches/%s/%s' % \
+                            (quote(tag.category), quote(str(tag.id)))
+                            vals.append(tval.format(xml(tag.name, True),
+                                xml(href, True),
+                                xml(val if len(dbtags) == 1 else tag.name),
+                                xml(key, True)))
+                        join = ' &amp; ' if key == 'authors' else ', '
+                        args[key] = join.join(vals)
+                        added_key = True
+                if not added_key:
+                    args[key] = xml(val, True)
+            else:
+                args[key] = xml(val, True)
         fname = quote(ascii_filename(args['title']) + ' - ' +
                 ascii_filename(args['authors']))
         return args, fmt, fmts, fname
@@ -674,7 +776,8 @@ class BrowseServer(object):
         except:
             return _('This book has been deleted')
         else:
-            args, fmt, fmts, fname = self.browse_get_book_args(mi, id_)
+            args, fmt, fmts, fname = self.browse_get_book_args(mi, id_,
+                    add_category_links=True)
             args['formats'] = ''
             if fmts:
                 ofmts = [u'<a href="{4}/get/{0}/{1}_{2}.{0}" title="{3}">{3}</a>'\
@@ -690,8 +793,9 @@ class BrowseServer(object):
                 if m['is_custom'] and field not in displayed_custom_fields:
                     continue
                 if m['datatype'] == 'comments' or field == 'comments':
-                    comments.append((m['name'], comments_to_html(mi.get(field,
-                        ''))))
+                    val = mi.get(field, '')
+                    if val and val.strip():
+                        comments.append((m['name'], comments_to_html(val)))
                     continue
                 if field in ('title', 'formats') or not args.get(field, False) \
                         or not m['name']:
